/*
 * FMATHADD is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2001 by Michael Schroeder <mdhs@gmx.net>
 */

#ifndef	NO_IMATH		/* Inline-Funktionen benutzen ? */
# define IMATH_COMPLEX
#endif


#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fmath.h"
#include "fmathctl.h"
#include "fcomplex.h"
#include "starcalc.h"
#include "locals.h"


/*
 * String in komplexe Zahl zerlegen. Zur Kennzeichnung des komplexen Teils wird
 * der Buchstabe i benutzt. (a + bi)
 *
 * RC: FALSE (0) wenn Fehler;  TRUE wenn OK (z enthaelt die komplexe Zahl)
 *
 * Unterstuetzte Formate:	(# = Zahl)
 *
 *	[+-]#
 *	[+-]#i	
 *	[+-]i
 *	[+-]# <[+-]>#i
 *	[+-]# <[+-]>i
 *
 *	(i darf auch als j geschrieben werden)
 */
static int getxnum(char *str, COMPLEX *z)
{
  char *ptr = str, *end;
  double d;

  *z = 0;			/* mit komplexer 0 vorbelegen */

  SKIPSPACE(ptr);		/* Zeiger auf erstes Zeichen */

  d = strtod(ptr, &end);		/* umwandeln */

  if (ptr == end) {		/* Umwandlung nicht moeglich ? */

    switch (*(unsigned short*)(ptr)) {

      case MKNUM('i', '\0'):		/* i */
      case MKNUM('j', '\0'):		/* j */
	IMAG *z = 1;
	return TRUE;

      case MKNUM('i', ' '):		/* i  */
      case MKNUM('j', ' '):		/* j  */
      case MKNUM('+', 'i'):		/* +i */
      case MKNUM('+', 'j'):		/* +j */
        IMAG *z = 1;	
	break;

      case MKNUM('-', 'i'):		/* -i */
      case MKNUM('-', 'j'):		/* -j */
        IMAG *z = -1;
	break;

      default:				/* Fehler */
        return FALSE;
    }
    ptr += 2;				/* auf naechstes Zeichen */

    SKIPSPACE(ptr);			/* nur Leerzeichen sind erlaubt...*/

    return (*ptr == 0)? TRUE: FALSE; 	/* ...andere Zeichen = Fehler */
  }
  else {				/* Umwandlung war erfolgreich! */

    if (*end == 'i' || *end == 'j') {	/* welcher Teil? */

      end++;

      IMAG *z = d;			/* eintragen */

      SKIPSPACE(end);			/* nur Leerzeichen sind erlaubt...*/

      return (*end == 0)? TRUE: FALSE;	/* ...andere Zeichen = Fehler */
    }
    else {

      REAL *z = d;

      if (*end == 0)			/* nur eine normale Zahl? */ 
        return TRUE;
    }
    ptr = end;				/* auf naechstes Zeichen */
  }
			/* Spielchen wiederholen */

  SKIPSPACE(ptr);			/* Zeiger auf erstes Zeichen */

  d = strtod(ptr, &end);		/* umwandeln */

  if (ptr == end) {		/* Umwandlung nicht moeglich ? */

    switch (*(unsigned short*)(ptr)) {

      case MKNUM('+', 'i'):		/* +i */
      case MKNUM('+', 'j'):		/* +j */
        IMAG *z = 1;	
	break;

      case MKNUM('-', 'i'):		/* -i */
      case MKNUM('-', 'j'):		/* -j */
        IMAG *z = -1;
	break;

      default:				/* Fehler */
        return FALSE;
    }
    ptr += 2;				/* auf naechstes Zeichen */

    SKIPSPACE(ptr);			/* nur Leerzeichen sind erlaubt...*/

    return (*ptr == 0)? TRUE: FALSE; 	/* ...andere Zeichen = Fehler */
  }
  else {				/* Umwandlung war erfolgreich! */

    if (*end == 'i' || *end == 'j') {	/* welcher Teil? */

      end++;

      IMAG *z = d;			/* eintragen */

      SKIPSPACE(end);			/* nur Leerzeichen sind erlaubt...*/

      return (*end == 0)? TRUE: FALSE;	/* ...andere Zeichen = Fehler */
    }
    else
      return FALSE;			/* noch ein Real-Teil? */ 
  }

  return TRUE;
}


/*
 * Komplexes Ergebnis formatieren und zurueckliefern
 */
static void retxnum(char *r, COMPLEX *z)
{
  char buf[256];
  double a;

  if (!_isfinit(REAL *z) || !_isfinit(IMAG *z)) {
    ERR_CAL(r);
    return;
  } 

  if ((a = xabs(*z)) == 0) {		/* *z = 0 ? */
    strcpy(r, "0");
    return;
  }

  if ((fabs(IMAG *z) / a) < EPSILON)	/* Runden */
    IMAG *z = 0;
  else if ((fabs(REAL *z) / a) < EPSILON)
    REAL *z = 0;

  if (IMAG *z == 0) {			/* Formatieren */
    sprintf(buf, "%.15G", REAL *z);
  }
  else if (REAL *z == 0) {
    sprintf(buf, "%.15Gi", IMAG *z);
  }
  else
    sprintf(buf, "%.15G%+.15Gi", REAL *z, IMAG *z);

  SO_StringCopy(r, buf);
}


/*
 * Realteil berechnen
 */
SO_CALLBACK(SC_xreal, double *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    *r = INVALID;
    return;
  }

  *r = REAL z;
}


/*
 * Imaginearteil berechnen
 */
SO_CALLBACK(SC_ximag, double *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    *r = INVALID;
    return;
  }

  *r = IMAG z;
}


/*
 * Absolutwert berechnen
 */
SO_CALLBACK(SC_xabs, double *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    *r = INVALID;
    return;
  }

  *r = xabs(z);
}


/*
 * Argument berechnen
 */
SO_CALLBACK(SC_xarg, double *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    *r = INVALID;
    return;
  }

  *r = xarg(z);
}


/*
 * Konjugiert-komplex
 */
SO_CALLBACK(SC_xconj, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = CONJ(z);

  retxnum(r, &z);
}


/*
 * Addition
 */
SO_CALLBACK(SC_xadd, char *r, char *a, char *b)
{
  COMPLEX x, y;

  if (!getxnum(a, &x)) {		/* 1. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  if (!getxnum(b, &y)) {		/* 2. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  x += y;		/* addieren */

  retxnum(r, &x);
}


/*
 * Subtraktion
 */
SO_CALLBACK(SC_xsub, char *r, char *a, char *b)
{
  COMPLEX x, y;

  if (!getxnum(a, &x)) {		/* 1. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  if (!getxnum(b, &y)) {		/* 2. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  x -= y;		/* subtrahieren */

  retxnum(r, &x);
}


/*
 * Multiplikation
 */
SO_CALLBACK(SC_xmul, char *r, char *a, char *b)
{
  COMPLEX x, y;

  if (!getxnum(a, &x)) {		/* 1. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  if (!getxnum(b, &y)) {		/* 2. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  x *= y;		/* multiplizieren */

  retxnum(r, &x);
}


/*
 * Division
 */
SO_CALLBACK(SC_xdiv, char *r, char *a, char *b)
{
  COMPLEX x, y;

  if (!getxnum(a, &x)) {		/* 1. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  if (!getxnum(b, &y)) {		/* 2. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  x /= y;		/* dividieren */

  retxnum(r, &x);
}


/*
 * Quadratwurzel
 */
SO_CALLBACK(SC_xsqrt, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xsqrt(z);

  retxnum(r, &z);
}


/*
 * Potenz
 */
SO_CALLBACK(SC_xpow, char *r, char *a, char *b)
{
  COMPLEX x, y;

  if (!getxnum(a, &x)) {		/* 1. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  if (!getxnum(b, &y)) {		/* 2. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  x = xpow(x, y);		/* potenzieren */

  retxnum(r, &x);
}


/*
 * e-Funktion
 */
SO_CALLBACK(SC_xexp, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xexp(z);

  retxnum(r, &z);
}


/*
 * Dekadische Potenz
 */
SO_CALLBACK(SC_xpow10, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xpow10(z);

  retxnum(r, &z);
}


/*
 * Freier Logarithmus
 */
SO_CALLBACK(SC_xlogy, char *r, char *a, char *b)
{
  COMPLEX x, y;

  if (!getxnum(a, &x)) {		/* 1. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  if (!getxnum(b, &y)) {		/* 2. komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  x = xlogy(x, y);		/* potenzieren */

  retxnum(r, &x);
}


/*
 * Natuerlicher Logarithmus
 */
SO_CALLBACK(SC_xlog, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xlog(z);

  retxnum(r, &z);
}


/*
 * Dekadischer Logarithmus
 */
SO_CALLBACK(SC_xlog10, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xlog10(z);

  retxnum(r, &z);
}


/*
 * Sinus
 */
SO_CALLBACK(SC_xsin, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xsin(z);

  retxnum(r, &z);
}


/*
 * Cosinus
 */
SO_CALLBACK(SC_xcos, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xcos(z);

  retxnum(r, &z);
}


/*
 * Tangens
 */
SO_CALLBACK(SC_xtan, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xtan(z);

  retxnum(r, &z);
}


/*
 * Sinus hyperbolicus
 */
SO_CALLBACK(SC_xsinh, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xsinh(z);

  retxnum(r, &z);
}


/*
 * Cosinus hyperbolicus
 */
SO_CALLBACK(SC_xcosh, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xcosh(z);

  retxnum(r, &z);
}


/*
 * Tangens hyperbolicus
 */
SO_CALLBACK(SC_xtanh, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xtanh(z);

  retxnum(r, &z);
}


/*
 * Arcus sinus
 */
SO_CALLBACK(SC_xasin, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xasin(z);

  retxnum(r, &z);
}


/*
 * Arcus cosinus
 */
SO_CALLBACK(SC_xacos, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xacos(z);

  retxnum(r, &z);
}


/*
 * Arcus tangens
 */
SO_CALLBACK(SC_xatan, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xatan(z);

  retxnum(r, &z);
}


/*
 * Area sinus hyperbolicus
 */
SO_CALLBACK(SC_xasinh, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xasinh(z);

  retxnum(r, &z);
}


/*
 * Area cosinus hyperbolicus
 */
SO_CALLBACK(SC_xacosh, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xacosh(z);

  retxnum(r, &z);
}


/*
 * Area tangens hyperbolicus
 */
SO_CALLBACK(SC_xatanh, char *r, char *a)
{
  COMPLEX z;

  if (!getxnum(a, &z)) {		/* komplexe Zahl holen */
    ERR_ARG(r);
    return;
  }

  z = xatanh(z);

  retxnum(r, &z);
}


/*
 * Die Summes der komplexen Zahlen eines Zellgebiets
 */
SO_CALLBACK(SC_xsum, char *r, CELLARRAY *array)
{
  int count = array->count;		/* Anzahl der Zellen */
  COMPLEX c, z = 0;
  SUBCELL *cell = &array->first;	/* Zeiger auf 1. Zelle */

  while (count--) {			/* alle Zellen durchlaufen */

    if (cell->error) {			/* bei Fehlern abbrechen */
      ERR_ARG(r);
      return;
    }

    if (cell->type == CELL_NUMBER) {	/* normale Zahl? */

      z += cell->var.value;		/* summieren */

      cell++;				/* naechste Zelle */

    }
    else {				/* komplexe Zahl */

      if (!getxnum(cell->var.string.text, &c)) {
	ERR_ARG(r);
	return;
      }

      z += c;				/* summieren */

      NEXTCELL(cell);			/* naechste Zelle */

    }
  }

  retxnum(r, &z);
}










