/* $Id$ */
/*-----------------------------------------------------*/
/*     Name:             fcomplex.h                    */
/*-----------------------------------------------------*/
/*  Aufgabe: Komplexe mathematische Funktionen         */
/*-----------------------------------------------------*/
/*   Copyright (C) 1995 - 2000 by Michael Schroeder    */
/*-----------------------------------------------------*/


/*
 * LIBFMATH is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 */


#ifndef FCOMPLEX_H
 #define FCOMPLEX_H

#if !__cplusplus && __GNUC__	/***** Komplexe Zahlen nur fuer GNU C! *****/

#define CONJ(x)		~x		/* Konjugiert-komplex */
#define REAL		__real__
#define IMAG		__imag__

#define _COMPLEX	COMPLEX		/* vorlaeufig */

typedef __complex__ float		 fCOMPLEX,  *pfCOMPLEX;
typedef __complex__ double		  COMPLEX,   *pCOMPLEX;
typedef __complex__ long double		ldCOMPLEX, *pldCOMPLEX;


float xabsf(fCOMPLEX z);
float xargf(fCOMPLEX z);
_COMPLEX xacosf(fCOMPLEX z);
_COMPLEX xacoshf(fCOMPLEX z);
_COMPLEX xasinf(fCOMPLEX z);
_COMPLEX xasinhf(fCOMPLEX z);
_COMPLEX xatanf(fCOMPLEX z);
_COMPLEX xatanhf(fCOMPLEX z);
_COMPLEX xcosf(fCOMPLEX z);
_COMPLEX xcoshf(fCOMPLEX z);
_COMPLEX xexpf(fCOMPLEX z);
_COMPLEX xlogf(fCOMPLEX z);
_COMPLEX xlogyf(fCOMPLEX a, fCOMPLEX b);
_COMPLEX xlog10f(fCOMPLEX z);
_COMPLEX xlog2f(fCOMPLEX z);
_COMPLEX xpowf(fCOMPLEX a, fCOMPLEX b);
_COMPLEX xpow10f(fCOMPLEX z);
_COMPLEX xpow2f(fCOMPLEX z);
_COMPLEX xsinf(fCOMPLEX z);
_COMPLEX xsinhf(fCOMPLEX z);
_COMPLEX xsqrtf(fCOMPLEX z);
_COMPLEX xtanf(fCOMPLEX z);
_COMPLEX xtanhf(fCOMPLEX z);

double xabs(COMPLEX z);
double xarg(COMPLEX z);
COMPLEX xacos(COMPLEX z);
COMPLEX xacosh(COMPLEX z);
COMPLEX xasinh(COMPLEX z);
COMPLEX xasin(COMPLEX z);
COMPLEX xatan(COMPLEX z);
COMPLEX xatanh(COMPLEX z);
COMPLEX xcos(COMPLEX z);
COMPLEX xcosh(COMPLEX z);
COMPLEX xexp(COMPLEX z);
COMPLEX xlog(COMPLEX z);
COMPLEX xlogy(COMPLEX a, COMPLEX b);
COMPLEX xlog10(COMPLEX z);
COMPLEX xlog2(COMPLEX z);
COMPLEX xpow(COMPLEX a, COMPLEX b);
COMPLEX xpow10(COMPLEX z);
COMPLEX xpow2(COMPLEX z);
COMPLEX xsin(COMPLEX z);
COMPLEX xsinh(COMPLEX z);
COMPLEX xsqrt(COMPLEX z);
COMPLEX xtan(COMPLEX z);
COMPLEX xtanh(COMPLEX z);

long double xabsl(ldCOMPLEX z);
long double xargl(ldCOMPLEX z);
ldCOMPLEX xacosl(ldCOMPLEX z);
ldCOMPLEX xacoshl(ldCOMPLEX z);
ldCOMPLEX xasinl(ldCOMPLEX z);
ldCOMPLEX xasinhl(ldCOMPLEX z);
ldCOMPLEX xatanl(ldCOMPLEX z);
ldCOMPLEX xatanhl(ldCOMPLEX z);
ldCOMPLEX xcosl(ldCOMPLEX z);
ldCOMPLEX xcoshl(ldCOMPLEX z);
ldCOMPLEX xexpl(ldCOMPLEX z);
ldCOMPLEX xlogl(ldCOMPLEX z);
ldCOMPLEX xlogyl(ldCOMPLEX a, ldCOMPLEX b);
ldCOMPLEX xlog10l(ldCOMPLEX z);
ldCOMPLEX xlog2l(ldCOMPLEX z);
ldCOMPLEX xpowl(ldCOMPLEX a, ldCOMPLEX b);
ldCOMPLEX xpow10l(ldCOMPLEX z);
ldCOMPLEX xpow2l(ldCOMPLEX z);
ldCOMPLEX xsinl(ldCOMPLEX z);
ldCOMPLEX xsinhl(ldCOMPLEX z);
ldCOMPLEX xsqrtl(ldCOMPLEX z);
ldCOMPLEX xtanl(ldCOMPLEX z);
ldCOMPLEX xtanhl(ldCOMPLEX z);


/*
 * Die folgenden Inline-Funktionen werden verwendet, falls
 * folgendes definiert wird: #define IMATH_COMPLEX
 */
#ifdef IMATH_COMPLEX

#ifndef fast
 #ifdef	__cplusplus
  #define fast		__inline__
 #else
  #define fast		extern __inline__
 #endif
#endif

#ifndef mcode
 #define mcode		__asm__ __volatile__
#endif

fast float xargf(fCOMPLEX z)
{
   register float a, re = REAL z, im = IMAG z;

   mcode ("fpatan": "=t" (a): "u" (im), "0" (re): "st", "st(1)");    

   return a;
}

fast double xarg(COMPLEX z)
{
   register double a, re = REAL z, im = IMAG z;

   mcode ("fpatan": "=t" (a): "u" (im), "0" (re): "st", "st(1)");    

   return a;
}

fast long double xargl(ldCOMPLEX z)
{
   register long double a, re = REAL z, im = IMAG z;

   mcode ("fpatan": "=t" (a): "u" (im), "0" (re): "st", "st(1)");    

   return a;
}

fast float xabsf(fCOMPLEX z)
{
   register float a, c = REAL z * REAL z + IMAG z * IMAG z;

   mcode ("fsqrt": "=t" (a): "0" (c));
    
   return a;
}

fast double xabs(COMPLEX z)
{
   register double a, c = REAL z * REAL z + IMAG z * IMAG z;

   mcode ("fsqrt": "=t" (a): "0" (c));

   return a;
}

fast long double xabsl(ldCOMPLEX z)
{
   register long double a, c = REAL z * REAL z + IMAG z * IMAG z;

   mcode ("fsqrt": "=t" (a): "0" (c));
    
   return a;
}


#endif	/* IMATH_COMPLEX */

#endif	/* !__cplusplus && __GNUC__ */


#endif		/* FCOMPLEX_H */

