/*
 * FMATHADD is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2001 by Michael Schroeder <mdhs@gmx.net>
 */


#define INCL_NLANGUAGE

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fmath.h"
#include "starcalc.h"
#include "locals.h"



SO_CALLBACK(GetFunctionCount, USHORT *nCount)
{
  setlocale(LC_NUMERIC, "");		/* Dezimalpunkt einstellen */
  
  *nCount = COUNT_OF_FUNCS;
}


/*
 * Hier folgen kurze Erlaeuterungen zu den Funktionen und Argumenten
 * in verschiedenen Sprachen (auf richtigen Zeichensatz achten) 
 */
static char *getText(int nResource)
{
  switch(nResource) {

    EXPLAIN(NUMBER_NAME,
	    "Number",
	    "Zahl");
    EXPLAIN(NUMBER_DESC,
	    "A number",
	    "Eine Zahl");
    EXPLAIN(COMPLEX_NAME,
	    "complex number",
	    "komplexe Zahl");
    EXPLAIN(COMPLEX_DESC,
	    "A complex number: a+bi",
	    "Eine komplexe Zahl: a+bi");
    EXPLAIN(DECIMAL_NAME,
	    "Decimals",
	    "Nachkommastellen");
    EXPLAIN(DECIMAL_DESC,
	    "Number of displayed decimals",
	    "Zahl der angezeigten Nachkommastellen");
    EXPLAIN(COEFF_NAME,
	    "Coefficients",
	    "Koeffizienten");
    EXPLAIN(COEFF_DESC,
	    "cell array of coefficients",
	    "Zellbereich mit Koeffizienten");
    EXPLAIN(ORDER_NAME,
	    "Order",
	    "Ordnung");
    EXPLAIN(ORDER_DESC,
	    "Order of the function",
	    "Ordnung der Funktion");
    EXPLAIN(ROOT_NAME,
	    "Root",
	    "Wurzel");
    EXPLAIN(ROOT_DESC,
	    "Value of the root (0 for a zero)",
	    "Wert der Wurzel (0 f"ue"r eine Nullstelle)");
    EXPLAIN(ARRAY_NAME,
	    "Cell array",
	    "Zellbereich");
    EXPLAIN(ARRAY_DESC,
	    "Cell array with table of numbers",
	    "Zellbereich mit Zahlentabelle");
    EXPLAIN(ROMANS_NAME,
	    "Roman number",
	    "R"oe"mische Ziffer");
    EXPLAIN(ROMANS_DESC,
	    "String with the roman number",
	    "Zeichenkette mit der r"oe"mischen Zahl");
/*
    EXPLAIN(XARRAY_NAME,
	    "Complex number array",
	    "Komplexzahlen-Bereich");
*/
    EXPLAIN(XARRAY_DESC,
	    "Cell array with table of complex numbers",
	    "Zellbereich mit komplexer Zahlentabelle");

    EXPLAIN(GAMMA_NAME,
	    "GAMMA",
	    "GAMMA");
    EXPLAIN(GAMMA_DESC,
	    "Calculetes the gamma function",
	    "Berechnet die Gamma-Funktion");
    EXPLAIN(BESSEL_NAME,
	    "BESSEL",
	    "BESSEL");
    EXPLAIN(BESSEL_DESC,
	    "Calculetes the Bessel function of the 1st kind of order n",
	    "Berechnet die Besselsche Funktion 1. Gattung n-ter Ordnung");
    EXPLAIN(WEBER_NAME,
	    "WEBER",
	    "WEBER");
    EXPLAIN(WEBER_DESC,
	    "Calculetes the Weber function (Bessel function of the 2nd kind "
	    "of order n)",
	    "Berechnet die Webersche Funktion (Besselsche Funktion 2. Gattung "
	    " n-ter Ordnung)");

    EXPLAIN(XREAL_NAME,
	    "REAL",
	    "REAL");
    EXPLAIN(XREAL_DESC,
	    "Determines the real part of the complex number",
	    "Bestimmt den Realteil der komplexen Zahl");
    EXPLAIN(XIMAG_NAME,
	    "IMAG",
	    "IMAG");
    EXPLAIN(XIMAG_DESC,
	    "Determines the imaginary part of the complex number",
	    "Bestimmt den Imagin"ae"rteil der komplexen Zahl");


    EXPLAIN(XABS_NAME,
	    "ABSOLUTE",
	    "ABSOLUT");
    EXPLAIN(XABS_DESC,
	    "Determines the absolute value of the complex number",
	    "Bestimmt den Absolutbetrag der komplexen Zahl");
    EXPLAIN(XARG_NAME,
	    "ARGUMENT",
	    "ARGUMENT");
    EXPLAIN(XARG_DESC,
	    "Determines the argument of the complex number",
	    "Bestimmt das Argument der komplexen Zahl");

    EXPLAIN(XCONJ_NAME,
	    "CONJUGATE",
	    "KONJUGIERT");
    EXPLAIN(XCONJ_DESC,
	    "Performs complex conjugation of the complex number",
	    "Bestimmt die konjugiert komplexe Zahl");

    EXPLAIN(XADD_NAME,
	    "XADD",
	    "XADD");
    EXPLAIN(XADD_DESC,
	    "Performs a complex addition",
	    "Addiert die komplexen Zahlen");
    EXPLAIN(XSUB_NAME,
	    "XSUB",
	    "XSUB");
    EXPLAIN(XSUB_DESC,
	    "Performs a complex subtraction",
	    "Subtrahiert die komplexen Zahlen");
    EXPLAIN(XMUL_NAME,
	    "XMUL",
	    "XMUL");
    EXPLAIN(XMUL_DESC,
	    "Performs a complex mutiplication",
	    "Multipliziert die komplexen Zahlen");
    EXPLAIN(XDIV_NAME,
	    "XDIV",
	    "XDIV");
    EXPLAIN(XDIV_DESC,
	    "Performs a complex division",
	    "Dividiert die komplexen Zahlen");

    EXPLAIN(XSQRT_NAME,
	    "XSQRT",
	    "XWURZEL");
    EXPLAIN(XSQRT_DESC,
	    "Calculates the complex square root",
	    "Berechnet die Quadratwurzel der komplexen Zahl");

    EXPLAIN(XPOW_NAME,
	    "XPOWER",
	    "XPOTENZ");
    EXPLAIN(XPOW_DESC,
	    "Calculates the complex power",
	    "Berechnet die Potenz der komplexen Zahlen");
    EXPLAIN(XEXP_NAME,
	    "XEXP",
	    "XEXP");
    EXPLAIN(XEXP_DESC,
	    "Calculates the complex exponentiation function",
	    "Berechnet die Exponentialfunktion der komplexen Zahl");

    EXPLAIN(XPOW10_NAME,
	    "XPOW10",
	    "XPOW10");
    EXPLAIN(XPOW10_DESC,
	    "Calculates the complex power of 10",
	    "Berechnet die 10er Potenz der komplexen Zahl");

    EXPLAIN(XLOGY_NAME,
	    "XLOG",
	    "XLOG");
    EXPLAIN(XLOGY_DESC,
	    "Calculates the complex logarithm",
	    "Berechnet den Logarithmus der komplexen Zahlen");

    EXPLAIN(XLOG_NAME,
	    "XLN",
	    "XLN");
    EXPLAIN(XLOG_DESC,
	    "Calculates the complex natural logarithm",
	    "Berechnet den nat"ue"rlichen Logarithmus der komplexen Zahl");

    EXPLAIN(XLOG10_NAME,
	    "XLOG10",
	    "XLOG10");
    EXPLAIN(XLOG10_DESC,
	    "Calculates the complex base-10 logarithm",
	    "Berechnet den dekadischen Logarithmus der komplexen Zahl");

    EXPLAIN(XSIN_NAME,
	    "XSIN",
	    "XSIN");
    EXPLAIN(XSIN_DESC,
	    "Calculates the complex sine",
	    "Berechnet den Sinus der komplexen Zahl");
    EXPLAIN(XCOS_NAME,
	    "XCOS",
	    "XCOS");
    EXPLAIN(XCOS_DESC,
	    "Calculates the complex cosine",
	    "Berechnet den Kosinus der komplexen Zahl");
    EXPLAIN(XTAN_NAME,
	    "XTAN",
	    "XTAN");
    EXPLAIN(XTAN_DESC,
	    "Calculates the complex tangent",
	    "Berechnet den Tangens der komplexen Zahl");

    EXPLAIN(XSINH_NAME,
	    "XSINHYP",
	    "XSINHYP");
    EXPLAIN(XSINH_DESC,
	    "Calculates the complex hyperbolic sine",
	    "Berechnet den Sinus Hyperbolicus der komplexen Zahl");
    EXPLAIN(XCOSH_NAME,
	    "XCOSHYP",
	    "XCOSHYP");
    EXPLAIN(XCOSH_DESC,
	    "Calculates the complex hyperbolic cosine",
	    "Berechnet den Kosinus Hyperbolicus der komplexen Zahl");
    EXPLAIN(XTANH_NAME,
	    "XTANHYP",
	    "XTANHYP");
    EXPLAIN(XTANH_DESC,
	    "Calculates the complex hyperbolic tangent",
	    "Berechnet den Tangens Hyperbolicus der komplexen Zahl");

    EXPLAIN(XASIN_NAME,
	    "XARCSIN",
	    "XARCSIN");
    EXPLAIN(XASIN_DESC,
	    "Calculates the complex arc sine",
	    "Berechnet den Arkussinus der komplexen Zahl");
    EXPLAIN(XACOS_NAME,
	    "XARCCOS",
	    "XARCCOS");
    EXPLAIN(XACOS_DESC,
	    "Calculates the complex arc cosine",
	    "Berechnet den Arkuskosinus der komplexen Zahl");
    EXPLAIN(XATAN_NAME,
	    "XARCTAN",
	    "XARCTAN");
    EXPLAIN(XATAN_DESC,
	    "Calculates the complex arc tangent",
	    "Berechnet den Arkustangens der komplexen Zahl");

    EXPLAIN(XASINH_NAME,
	    "XARCSINHYP",
	    "XARCSINHYP");
    EXPLAIN(XASINH_DESC,
	    "Calculates the complex inverse hyperbolic sine",
	    "Berechnet den inversen Sinus Hyperbolicus der komplexen Zahl");
    EXPLAIN(XACOSH_NAME,
	    "XARCCOSHYP",
	    "XARCCOSHYP");
    EXPLAIN(XACOSH_DESC,
	    "Calculates the complex inverse hyperbolic cosine",
	    "Berechnet den inversen Kosinus Hyperbolicus der komplexen Zahl");
    EXPLAIN(XATANH_NAME,
	    "XARCTANHYP",
	    "XARCTANHYP");
    EXPLAIN(XATANH_DESC,
	    "Calculates the complex inverse hyperbolic tangent",
	    "Berechnet den inversen Tangens Hyperbolicus der komplexen Zahl");

    EXPLAIN(XSUM_NAME,
	    "XSUM",
	    "XSUMME");
    EXPLAIN(XSUM_DESC,
	    "Calculates the sum of the complex numbers",
	    "Berechnet die Summe der komplexen Zahlen");

    EXPLAIN(EURO_NAME,
	    "EURO",
	    "EURO");
    EXPLAIN(EURO_DESC,
	    "Formates the number as Euro currency",
	    "Formatiert die Zahl als Euro-W"ae"hrung");
    EXPLAIN(IEEE_NAME,
	    "IEEE",
	    "IEEE");
    EXPLAIN(IEEE_DESC,
	    "Converts the number to IEEE floating point format",
	    "Wandelt die Zahl in das IEEE-Flie"ss"kommaformat um");
    EXPLAIN(ROMAN_NAME,
	    "ROMANTONUM",
	    "R"OE"MISCHINZAHL");
    EXPLAIN(ROMAN_DESC,
	    "Converts the roman number to decimal number",
	    "Wandelt die r"oe"mische Ziffer in eine Dezimalzahl um");

    EXPLAIN(IPFUNC_NAME,
	    "IPOLVALUE",
	    "IPOLWERT");
    EXPLAIN(IPFUNC_DESC,
	    "Interpolation of the tabular values to the number",
	    "Interpolation der Tabellenwerten zur Zahl");
    EXPLAIN(IPROOT_NAME,
	    "IPOLINVERSE",
	    "IPOLINVERS");
    EXPLAIN(IPROOT_DESC,
	    "Inverse Interpolation of the tabular values to a function "
	    "value (zero) ",
	    "Inverse Interpolation der Tabellenwerten zum Funktionswert "
	    "(Nullstelle)");
    EXPLAIN(IPEXTR_NAME,
	    "IPOLEXTREMUM",
	    "IPOLEXTREMUM");
    EXPLAIN(IPEXTR_DESC,
	    "Interpolation of the tabular values to the extremum",
	    "Interpolation der Tabellenwerten zum Extremwert");
    EXPLAIN(LAGRANGE_NAME,
	    "LAGRANGE",
	    "LAGRANGE");
    EXPLAIN(LAGRANGE_DESC,
	    "Lagrange interpolation of the tabular values to the function "
	    "result of the number",
	    "Lagrange-Interpolation der Tabellenwerte zum Funktionswert "
	    "der Zahl");

    EXPLAIN(POLY_NAME,
	    "POLYNOMIAL",
	    "POLYNOM");
    EXPLAIN(POLY_DESC,
	    "Evaluates the polynomial of the number from the coefficients",
	    "Entwickelt das Polynom der Zahl aus den Koeffizienten");

    default:
  }

  return("");
}


SO_QUIRKSCALL(GetFunctionData, USHORT *nNo, char *pFuncName,
		USHORT *nParamCount, ParamType *peType, char *pInternalName)
{
  switch(*nNo) {

    case FUNC_GAMMA:
      SO_StringCopy(pFuncName, Q(SC_gamma));
      SO_StringCopy(pInternalName, getText(GAMMA_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      *nParamCount = 2;
      break;

    case FUNC_BESSEL:
      SO_StringCopy(pFuncName, Q(SC_bessel));
      SO_StringCopy(pInternalName, getText(BESSEL_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE;
      *nParamCount = 3;
      break;

    case FUNC_WEBER:
      SO_StringCopy(pFuncName, Q(SC_weber));
      SO_StringCopy(pInternalName, getText(WEBER_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE;
      *nParamCount = 3;
      break;

    case FUNC_XREAL:
      SO_StringCopy(pFuncName, Q(SC_xreal));
      SO_StringCopy(pInternalName, getText(XREAL_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XIMAG:
      SO_StringCopy(pFuncName, Q(SC_ximag));
      SO_StringCopy(pInternalName, getText(XIMAG_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XABS:
      SO_StringCopy(pFuncName, Q(SC_xabs));
      SO_StringCopy(pInternalName, getText(XABS_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XARG:
      SO_StringCopy(pFuncName, Q(SC_xarg));
      SO_StringCopy(pInternalName, getText(XARG_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XCONJ:
      SO_StringCopy(pFuncName, Q(SC_xconj));
      SO_StringCopy(pInternalName, getText(XCONJ_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XADD:
      SO_StringCopy(pFuncName, Q(SC_xadd));
      SO_StringCopy(pInternalName, getText(XADD_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_XSUB:
      SO_StringCopy(pFuncName, Q(SC_xsub));
      SO_StringCopy(pInternalName, getText(XSUB_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_XMUL:
      SO_StringCopy(pFuncName, Q(SC_xmul));
      SO_StringCopy(pInternalName, getText(XMUL_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_XDIV:
      SO_StringCopy(pFuncName, Q(SC_xdiv));
      SO_StringCopy(pInternalName, getText(XDIV_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_XSQRT:
      SO_StringCopy(pFuncName, Q(SC_xsqrt));
      SO_StringCopy(pInternalName, getText(XSQRT_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XPOW:
      SO_StringCopy(pFuncName, Q(SC_xpow));
      SO_StringCopy(pInternalName, getText(XPOW_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_XEXP:
      SO_StringCopy(pFuncName, Q(SC_xexp));
      SO_StringCopy(pInternalName, getText(XEXP_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XPOW10:
      SO_StringCopy(pFuncName, Q(SC_xpow10));
      SO_StringCopy(pInternalName, getText(XPOW10_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XLOGY:
      SO_StringCopy(pFuncName, Q(SC_xlogy));
      SO_StringCopy(pInternalName, getText(XLOGY_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      peType[2] = PTR_STRING;
      *nParamCount = 3;
      break;

    case FUNC_XLOG:
      SO_StringCopy(pFuncName, Q(SC_xlog));
      SO_StringCopy(pInternalName, getText(XLOG_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XLOG10:
      SO_StringCopy(pFuncName, Q(SC_xlog10));
      SO_StringCopy(pInternalName, getText(XLOG10_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XSIN:
      SO_StringCopy(pFuncName, Q(SC_xsin));
      SO_StringCopy(pInternalName, getText(XSIN_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XCOS:
      SO_StringCopy(pFuncName, Q(SC_xcos));
      SO_StringCopy(pInternalName, getText(XCOS_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XTAN:
      SO_StringCopy(pFuncName, Q(SC_xtan));
      SO_StringCopy(pInternalName, getText(XTAN_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XSINH:
      SO_StringCopy(pFuncName, Q(SC_xsinh));
      SO_StringCopy(pInternalName, getText(XSINH_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XCOSH:
      SO_StringCopy(pFuncName, Q(SC_xcosh));
      SO_StringCopy(pInternalName, getText(XCOSH_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XTANH:
      SO_StringCopy(pFuncName, Q(SC_xtanh));
      SO_StringCopy(pInternalName, getText(XTANH_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XASIN:
      SO_StringCopy(pFuncName, Q(SC_xasin));
      SO_StringCopy(pInternalName, getText(XASIN_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XACOS:
      SO_StringCopy(pFuncName, Q(SC_xacos));
      SO_StringCopy(pInternalName, getText(XACOS_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XATAN:
      SO_StringCopy(pFuncName, Q(SC_xatan));
      SO_StringCopy(pInternalName, getText(XATAN_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XASINH:
      SO_StringCopy(pFuncName, Q(SC_xasinh));
      SO_StringCopy(pInternalName, getText(XASINH_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XACOSH:
      SO_StringCopy(pFuncName, Q(SC_xacosh));
      SO_StringCopy(pInternalName, getText(XACOSH_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XATANH:
      SO_StringCopy(pFuncName, Q(SC_xatanh));
      SO_StringCopy(pInternalName, getText(XATANH_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_STRING;
      *nParamCount = 2;
      break;

    case FUNC_XSUM:
      SO_StringCopy(pFuncName, Q(SC_xsum));
      SO_StringCopy(pInternalName, getText(XSUM_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_CELL_ARR;
      *nParamCount=2;
      break;

    case FUNC_EURO:
      SO_StringCopy(pFuncName, Q(SC_euro));
      SO_StringCopy(pInternalName, getText(EURO_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE;
      *nParamCount=3;
      break;

    case FUNC_IEEE:
      SO_StringCopy(pFuncName, Q(SC_ieee));
      SO_StringCopy(pInternalName, getText(IEEE_NAME));
      peType[0] = PTR_STRING;
      peType[1] = PTR_DOUBLE;
      *nParamCount=2;
      break;

    case FUNC_ROMAN:
      SO_StringCopy(pFuncName, Q(SC_roman));
      SO_StringCopy(pInternalName, getText(ROMAN_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_STRING;
      *nParamCount=2;
      break;

    case FUNC_IPFUNC:
      SO_StringCopy(pFuncName, Q(SC_ipfunc));
      SO_StringCopy(pInternalName, getText(IPFUNC_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE_ARR;
      *nParamCount=3;
      break;

    case FUNC_IPROOT:
      SO_StringCopy(pFuncName, Q(SC_iproot));
      SO_StringCopy(pInternalName, getText(IPROOT_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE_ARR;
      *nParamCount=3;
      break;

    case FUNC_IPEXTR:
      SO_StringCopy(pFuncName, Q(SC_ipextr));
      SO_StringCopy(pInternalName, getText(IPEXTR_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE_ARR;
      *nParamCount=2;
      break;

    case FUNC_LAGRANGE:
      SO_StringCopy(pFuncName, Q(SC_lagrange));
      SO_StringCopy(pInternalName, getText(LAGRANGE_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE_ARR;
      *nParamCount=3;
      break;

    case FUNC_POLY:
      SO_StringCopy(pFuncName, Q(SC_poly));
      SO_StringCopy(pInternalName, getText(POLY_NAME));
      peType[0] = PTR_DOUBLE;
      peType[1] = PTR_DOUBLE;
      peType[2] = PTR_DOUBLE_ARR;
      *nParamCount=3;
      break;

    default:
      *nParamCount = *pFuncName = *pInternalName = 0;
  }
}


/*
 * Jede exportierte Funktion erhaelt eine kurze Erlaeuterung was sie tut,
 * welche Argumente sie erwartet usw...
 * Diese Kurzhilfe wird vom Funktionsautopiloten von StarCalc erwartet.
 *
 * Folgende Parameter muessen versorgt werden:
 *
 * USHORT *nNo:	   Nummer der Funktion zwischen 0 und nCount - 1  
 * USHORT *nParam: Nummer der Funktionsparameters
 * char   *pName:  Name des Funktionsparameters
 * char   *pDesc:  Kurze Beschreibung des Parameters
 */
SO_QUIRKSCALL(GetParameterDescription, USHORT* nNo, USHORT* nParam,
				      char* pName, char* pDesc)
{
  *pName = *pDesc = 0;

  switch (*nNo) {

    case FUNC_BESSEL:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(BESSEL_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(ORDER_NAME));
	  SO_StringCopy(pDesc, getText(ORDER_DESC));
	  break;
      }
      break;

    case FUNC_GAMMA:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(GAMMA_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;
      }
      break;

    case FUNC_WEBER:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(WEBER_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(ORDER_NAME));
	  SO_StringCopy(pDesc, getText(ORDER_DESC));
	  break;
      }
      break;

    case FUNC_XREAL:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XREAL_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XIMAG:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XIMAG_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XABS:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XABS_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XARG:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XARG_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XCONJ:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XCONJ_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XADD:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XADD_DESC));
	  break;

        case 1:
        case 2:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XSUB:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XSUB_DESC));
	  break;

        case 1:
        case 2:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XMUL:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XMUL_DESC));
	  break;

        case 1:
        case 2:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XDIV:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XDIV_DESC));
	  break;

        case 1:
        case 2:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XSQRT:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XSQRT_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XPOW:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XPOW_DESC));
	  break;

        case 1:
        case 2:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XEXP:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XEXP_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XPOW10:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XPOW10_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XLOGY:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XLOGY_DESC));
	  break;

        case 1:
        case 2:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XLOG:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XLOG_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XLOG10:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XLOG10_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XSIN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XSIN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XCOS:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XCOS_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XTAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XTAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XSINH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XSINH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XCOSH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XCOSH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XTANH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XTANH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XASIN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XASIN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XACOS:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XACOS_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XATAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XATAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XASINH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XASINH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XACOSH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XACOSH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XATANH:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XATANH_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(COMPLEX_NAME));
	  SO_StringCopy(pDesc, getText(COMPLEX_DESC));
	  break;
      }
      break;

    case FUNC_XSUM:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(XSUM_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(ARRAY_NAME));
	  SO_StringCopy(pDesc, getText(XARRAY_DESC));
	  break;
      }
      break;

    case FUNC_EURO:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(EURO_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(DECIMAL_NAME));
	  SO_StringCopy(pDesc, getText(DECIMAL_DESC));
	  break;
      }
      break;

    case FUNC_IEEE:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(IEEE_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;
      }
      break;

    case FUNC_ROMAN:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(ROMAN_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(ROMANS_NAME));
	  SO_StringCopy(pDesc, getText(ROMANS_DESC));
	  break;
      }
      break;

    case FUNC_LAGRANGE:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(LAGRANGE_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(ARRAY_NAME));
	  SO_StringCopy(pDesc, getText(ARRAY_DESC));
	  break;
      }
      break;

    case FUNC_IPFUNC:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(IPFUNC_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(ARRAY_NAME));
	  SO_StringCopy(pDesc, getText(ARRAY_DESC));
	  break;
      }
      break;

    case FUNC_IPROOT:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(IPROOT_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(ARRAY_NAME));
	  SO_StringCopy(pDesc, getText(ARRAY_DESC));
	  break;
      }
      break;

    case FUNC_IPEXTR:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(IPEXTR_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(ARRAY_NAME));
	  SO_StringCopy(pDesc, getText(ARRAY_DESC));
	  break;
      }
      break;

    case FUNC_POLY:
      switch (*nParam) {
        case 0:
	  SO_StringCopy(pDesc, getText(POLY_DESC));
	  break;

        case 1:
	  SO_StringCopy(pName, getText(NUMBER_NAME));
	  SO_StringCopy(pDesc, getText(NUMBER_DESC));
	  break;

        case 2:
	  SO_StringCopy(pName, getText(COEFF_NAME));
	  SO_StringCopy(pDesc, getText(COEFF_DESC));
	  break;
      }
      break;

  }
}
