/* $Id$ */
/*-----------------------------------------------------*/
/*     Name:             fmath.h                       */
/*-----------------------------------------------------*/
/*  Aufgabe: Funktionsprototypen und Strukturen        */
/*-----------------------------------------------------*/
/*   Copyright (C) 1995 - 2000 by Michael Schroeder    */
/*-----------------------------------------------------*/


/*
 * LIBFMATH is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 */


#ifndef FMATH_H
 #define FMATH_H

#if defined (__cplusplus)
extern "C" {
#endif

#define F_ZERO		 0.0
#define F_ONE		 1.0
#define F_TWO		 2.0
#define F_THREE		 3.0
#define F_FOUR		 4.0
#define F_FIVE		 5.0
#define F_SIX		 6.0
#define F_SEVEN		 7.0
#define F_EIGHT		 8.0
#define F_NINE		 8.0
#define F_C10		10.0
#define F_C60		60.0
#define F_C90		90.0
#define F_C100		100.0
#define F_C160		160.0
#define F_C180		180.0
#define F_C360		360.0

#define F_INFN		_infn				/* -Unendlich	*/
#define F_INFP		_infp				/* +Unendlich	*/
#define F_INF		F_INFP
#define F_QNAN		_qnan				/* Quiet NaN	*/
#define F_SNAN		_snan				/* Signaling NaN*/
#define _nan		_qnan

#define F_HALF		0.5
#define F_ONEAHALF	1.5
#define F_TWOAHALF	2.5
#define F_THREEAHALF	3.5
#define F_THIRD		.3333333333333333333333333
#define F_QUART		0.25
#define F_TENTH		0.1
#define F_SEXAG		1.666666666666666666666667E-2	/* 1/60		*/

#define F_SQRT2		1.414213562373095048801688	/* sqrt(2)	*/
#define F_SQRT1_2	.7071067811865475244008443	/* 1/sqrt(2)	*/

#define F_PI		3.141592653589793238462643	/* Pi		*/
#define F_TWOPI		6.283185307179586476925286	/* 2*Pi		*/
#define F_PI_2		1.570796326794896619231321	/* Pi/2		*/
#define F_PI_4		.7853981633974483096156608	/* Pi/4		*/
#define F_1_PI		.3183098861837906715377675	/* 1/Pi		*/
#define F_2_PI		.6366197723675813430755350	/* 2/Pi		*/
#define F_DEGREE	57.29577951308232087679815	/* 180/Pi	*/
#define F_GRAD		63.66197723675813430755351	/* 200/Pi	*/
#define F_RADIAN	1.745329251994329576923690E-2	/* Pi/180	*/
#define F_DARG		1.570796326794896619231322E-2	/* Pi/200	*/
#define F_RADIAN_2	8.726646259971647884618453E-3	/* Pi/360	*/
#define F_DARG_2	7.853981633974483096156608E-3	/* Pi/400	*/
#define F_SQRTPI	1.772453850905516027298167	/* sqrt(Pi)	*/
#define F_1_SQRTPI	.5641895835477562869480794	/* 1/sqrt(Pi)	*/
#define F_2_SQRTPI	1.128379167095512573896158	/* 2/sqrt(Pi)	*/

#define F_E		2.718281828459045235360287	/* e		*/
#define F_1_E		.3678794411714423215955237	/* 1/e		*/
#define F_LN10		2.302585092994045684017991	/* ln(10)	*/
#define F_LN2		.6931471805599453094172321	/* ln(2)	*/
#define F_LGE		.4342944819032518276511289	/* lg(e)	*/
#define F_LG2		.3010299956639811952137388	/* lg(2)	*/
#define F_LOG2E		1.442695040888963407359924	/* lb(e)	*/
#define F_2LOG2E	2.885390081777926814719849	/* 2*lb(e)	*/
#define F_LOG2E_2	.7213475204444817036799623	/* lb(e)/2	*/
#define F_LOG2T		3.321928094887362347870319	/* lb(10)	*/

#define F_EULER		.5772156649015328606065121	/* C		*/

#define F_EPSILON	1.0E-10				/* Abbruchgrenze */

#ifndef __ASSEMBLER__		/* fuer den GNU-Assembler ausblenden	*/

typedef struct			/* float SINCOS-Struktur	*/
{
   float cos;			/* Kosinus des Arguments */
   float sin;			/* Sinus des Arguments */
} fSINCOS, *pfSINCOS;
typedef const fSINCOS	cfSINCOS, *pcfSINCOS;

typedef struct			/* double SINCOS-Struktur	*/
{
   double cos;
   double sin;
} SINCOS, *pSINCOS;
typedef const SINCOS	cSINCOS, *pcSINCOS;

typedef struct			/* long double SINCOS-Struktur	*/
{
   long double cos;
   long double sin;
} ldSINCOS, *pldSINCOS;
typedef const ldSINCOS	cldSINCOS, *pcldSINCOS;

typedef struct			/* float rechtwinklige Koordinate	*/
{
   float x;
   float y;
} fRECTAN, *pfRECTAN,
  fPAIR, *pfPAIR;
typedef const fRECTAN	cfRECTAN, *pcfRECTAN, cfPAIR, *pcfPAIR;

typedef struct			/* double rechtwinklige Koordinate	*/
{
   double x;
   double y;
} RECTAN, *pRECTAN,
  PAIR, *pPAIR;
typedef const RECTAN	cRECTAN, *pcRECTAN, cPAIR, *pcPAIR;

typedef struct			/* long double rechtwinklige Koordinate	*/
{
   long double x;
   long double y;
} ldRECTAN, *pldRECTAN,
  ldPAIR, *pldPAIR;
typedef const ldRECTAN	cldRECTAN, *pcldRECTAN, cldPAIR, *pcldPAIR;

typedef struct			/* float kartesische Koordinate	*/
{
   float x;
   float y;
   float z;
} fCARTESIC, *pfCARTESIC,
  fSEXAGESI, *pfSEXAGESI,
  fVECTOR, *pfVECTOR;
typedef const fCARTESIC	cfCARTESIC, *pcfCARTESIC,
			cfSEXAGESI, *pcfSEXAGESI,
			cfVECTOR, *pcfVECTOR; 

typedef struct			/* double kartesische Koordinate	*/
{
   double x;
   double y;
   double z;
} CARTESIC, *pCARTESIC,
  SEXAGESI, *pSEXAGESI,
  VECTOR, *pVECTOR;
typedef const CARTESIC	cCARTESIC, *pcCARTESIC,
			cSEXAGESI, *pcSEXAGESI,
			cVECTOR, *pcVECTOR;

typedef struct			/* long double kartesische Koordinate	*/
{
   long double x;
   long double y;
   long double z;
} ldCARTESIC, *pldCARTESIC,
  ldSEXAGESI, *pldSEXAGESI,
  ldVECTOR, *pldVECTOR;
typedef const ldCARTESIC cldCARTESIC, *pcldCARTESIC,
			 cldSEXAGESI, *pcldSEXAGESI,
			 cldVECTOR, *pcldVECTOR;

typedef struct			/* float polare Koordinate	*/
{
   float r;
   float l;
} fPOLAR, *pfPOLAR;
typedef const fPOLAR	cfPOLAR, *pcfPOLAR;

typedef struct			/* double polare Koordinate	*/
{
   double r;
   double l;
} POLAR, *pPOLAR;
typedef const POLAR	cPOLAR, *pcPOLAR;

typedef struct			/* long double polare Koordinate	*/
{
   long double r;
   long double l;
} ldPOLAR, *pldPOLAR;
typedef const ldPOLAR	cldPOLAR, *pcldPOLAR;

typedef struct			/* float sphaerische Koordinate	*/
{
   float r;
   float l;
   float b;
} fSPHERIC, *pfSPHERIC;
typedef const fSPHERIC	cfSPHERIC, *pcfSPHERIC;

typedef struct			/* double sphaerische Koordinate	*/
{
   double r;
   double l;
   double b;
} SPHERIC, *pSPHERIC;
typedef const SPHERIC	cSPHERIC, *pcSPHERIC;

typedef struct			/* long double sphaerische Koordinate	*/
{
   long double r;
   long double l;
   long double b;
} ldSPHERIC, *pldSPHERIC;
typedef const ldSPHERIC	cldSPHERIC, *pcldSPHERIC;

typedef struct			/* float unbestimmter Vektor */
{
  int dim;			/* Dimension des Vektors, Polynoms...	*/
  float *coef;			/* Zeiger auf Koeffizienten		*/
} fUVECTOR, *pfUVECTOR,
  fUPOLYNOM, *pfUPOLYNOM;

typedef struct			/* double unbestimmter Vektor		*/
{
  int dim;			/* Dimension des Vektors, Polynoms...	*/
  double *coef;			/* Zeiger auf Koeffizienten		*/
} UVECTOR, *pUVECTOR,
  UPOLYNOM, *pUPOLYNOM;

typedef struct			/* long double unbestimmter Vektor */
{
  int dim;			/* Dimension des Vektors, Polynoms...	*/
  long double *coef;		/* Zeiger auf Koeffizienten		*/
} ldUVECTOR, *pldUVECTOR,
  ldUPOLYNOM, *pldUPOLYNOM;

typedef struct			/* float unbestimmte Matrix */
{
  int row;			/* Spaltenzahl				*/
  int col;			/* Zeilenzahl				*/
  float * coef;			/* Zeiger auf Koeffizienten M[row][col]	*/
} fUMATRIX, *pfUMATRIX;

typedef struct			/* double unbestimmte Matrix */
{
  int row;			/* Spaltenzahl				*/
  int col;			/* Zeilenzahl				*/
  double *coef;			/* Zeiger auf Koeffizienten M[row][col]	*/
} UMATRIX, *pUMATRIX;

typedef struct			/* long double unbestimmte Matrix */
{
  int row;			/* Spaltenzahl				*/
  int col;			/* Zeilenzahl				*/
  long double *coef;		/* Zeiger auf Koeffizienten M[row][col]	*/
} ldUMATRIX, *pldUMATRIX;

typedef struct			/* float 3x3-Matrix */
{
  float x00, x01, x02;
  float x10, x11, x12;
  float x20, x21, x22;
} fMATRIX, *pfMATRIX;
typedef const fMATRIX	cfMATRIX, *pcfMATRIX;

typedef struct			/* double 3x3-Matrix */
{
  double x00, x01, x02;
  double x10, x11, x12;
  double x20, x21, x22;
} MATRIX, *pMATRIX;
typedef const MATRIX	cMATRIX, *pcMATRIX;

typedef struct			/* long double 3x3-Matrix */
{
  long double x00, x01, x02;
  long double x10, x11, x12;
  long double x20, x21, x22;
} ldMATRIX, *pldMATRIX;
typedef const ldMATRIX	cldMATRIX, *pcldMATRIX;

typedef struct			/* float quadratisches Polynom		*/
{
   float x0;
   float x1;
   float x2;
} fQUAD, *pfQUAD;

typedef struct			/* double quadratisches Polynom		*/
{
   double x0;
   double x1;
   double x2;
} QUAD, *pQUAD;

typedef struct			/* long double quadratisches Polynom	*/
{
   long double x0;
   long double x1;
   long double x2;
} ldQUAD, *pldQUAD;

typedef struct			/* Sexagesimaler Winkel */
{
   long d;			/* Grad, Stunden, max. 9stellig */
   long m;			/* Minuten */
   long s;			/* Sekunden */
   long f;			/* Sekundenbruchteile, max. 9stellig */
} ANGLE, *pANGLE;

typedef struct				/* float Tschebyscheff-Polynom	*/
{
  int ord;				/* Ordnung der Entwicklung	*/
  float beg;				/* Start des Intervalls		*/
  float end;				/* Ende des Intervalls		*/
  void (*func)(float, void*);		/* Zu approximierende Funktion	*/
  void *coef;				/* Zeiger auf Koeffizientenarray*/
} fCHEBY, *pfCHEBY;

typedef struct				/* double Tschebyscheff-Polynom	*/
{
  int ord;				/* Ordnung der Entwicklung	*/
  double beg;				/* Start des Intervalls		*/
  double end;				/* Ende des Intervalls		*/
  void (*func)(double, void*);		/* Zu approximierende Funktion	*/
  void *coef;				/* Zeiger auf Koeffizientenarray*/
} CHEBY, *pCHEBY;

typedef struct			/* long double Tschebyscheff-Polynom	*/
{
  int ord;				/* Ordnung der Entwicklung	*/
  long double beg;			/* Start des Intervalls		*/
  long double end;			/* Ende des Intervalls		*/
  void (*func)(long double, void*);	/* Zu approximierende Funktion	*/
  void *coef;				/* Zeiger auf Koeffizientenarray*/
} ldCHEBY, *pldCHEBY;

typedef struct			/* float Regressionsanalyse	*/
{
   float cnt;			/* Anzahl der Elemente	*/
   float sumx;			/* Summe der x-Daten	*/
   float sumy;			/* Summe der y-Daten	*/
   float sumx2;			/* Summe der x^2-Daten	*/
   float sumy2;			/* Summe der y^2-Daten	*/
   float sumxy;			/* Summe der xy-Daten	*/
   float meanx;			/* Mittelw. der x-Daten	*/
   float meany;			/* Mittelw. der y-Daten	*/
   float sdxn;			/* Gesamt-x-Standardabw.*/
   float sdyn;			/* Gesamt-y-Standardabw.*/
   float sdx;			/* (n-1)-x-Standardabw.	*/
   float sdy;			/* (n-1)-x-Standardabw. */
   float cor;			/* Korrelationskoeff.	*/
   float lra;			/* Regress.-Konstante	*/
   float lrb;			/* Regress.-Koeffizient	*/
} fREGANALYSIS, *pfREGANALYSIS;

typedef struct			/* double Regressionsanalyse	*/
{
   double cnt;			/* Anzahl der Elemente	*/
   double sumx;			/* Summe der x-Daten	*/
   double sumy;			/* Summe der y-Daten	*/
   double sumx2;		/* Summe der x^2-Daten	*/
   double sumy2;		/* Summe der y^2-Daten	*/
   double sumxy;		/* Summe der xy-Daten	*/
   double meanx;		/* Mittelw. der x-Daten	*/
   double meany;		/* Mittelw. der y-Daten	*/
   double sdxn;			/* Gesamt-x-Standardabw.*/
   double sdyn;			/* Gesamt-y-Standardabw.*/
   double sdx;			/* (n-1)-x-Standardabw.	*/
   double sdy;			/* (n-1)-x-Standardabw. */
   double cor;			/* Korrelationskoeff.	*/
   double lra;			/* Regress.-Konstante	*/
   double lrb;			/* Regress.-Koeffizient	*/
} REGANALYSIS, *pREGANALYSIS;

typedef struct			/* long double Regressionsanalyse	*/
{
   long double cnt;		/* Anzahl der Elemente	*/
   long double sumx;		/* Summe der x-Daten	*/
   long double sumy;		/* Summe der y-Daten	*/
   long double sumx2;		/* Summe der x^2-Daten	*/
   long double sumy2;		/* Summe der y^2-Daten	*/
   long double sumxy;		/* Summe der xy-Daten	*/
   long double meanx;		/* Mittelw. der x-Daten	*/
   long double meany;		/* Mittelw. der y-Daten	*/
   long double sdxn;		/* Gesamt-x-Standardabw.*/
   long double sdyn;		/* Gesamt-y-Standardabw.*/
   long double sdx;		/* (n-1)-x-Standardabw.	*/
   long double sdy;		/* (n-1)-x-Standardabw. */
   long double cor;		/* Korrelationskoeff.	*/
   long double lra;		/* Regress.-Konstante	*/
   long double lrb;		/* Regress.-Koeffizient	*/
} ldREGANALYSIS, *pldREGANALYSIS;


/*
 * In RSXNT-DLLs (Win32) werden Daten als Zeiger auf den Inhalt uebergeben :-( 
 *
 * ( Anm.: Das funktioniert aber nur bei RSXNT-Programmen!!! )
 *
 * Durch die folgende Definition koennen die Daten importiert werden:
 *
 *  #define IMPORT_RSXNT_DLL_DATA
 */
#if defined(__RSXNT__) && defined(IMPORT_RSXNT_DLL_DATA)
# define _ieeemaxf	(*_ieeemaxf)
# define _ieeemax	(*_ieeemax)
# define _ieeemaxl	(*_ieeemaxl)
# define _ieeeminf	(*_ieeeminf)
# define _ieeemin	(*_ieeemin)
# define _ieeeminl	(*_ieeeminl)
# define _infp		(*_infp)
# define _infn		(*_infn)
# define _qnan		(*_qnan)
# define _snan		(*_snan)
# define _zero		(*_zero)
# define _one		(*_one)
# define _two		(*_two)
# define _three		(*_three)
# define _four		(*_four)
# define _five		(*_five)
# define _six		(*_six)
# define _seven		(*_seven)
# define _eight		(*_eight)
# define _nine		(*_nine)
# define _c10		(*_c10)
# define _c60		(*_c60)
# define _c90		(*_c90)
# define _c100		(*_c100)
# define _c160		(*_c160)
# define _c180		(*_c180)
# define _c360		(*_c360)
# define _half		(*_half)
# define _oneahalf	(*_oneahalf)
# define _twoahalf	(*_twoahalf)
# define _threeahalf	(*_threeahalf)
# define _third		(*_third)
# define _thirdf	(*_thirdf)
# define _thirdl	(*_thirdl)
# define _quart		(*_quart)
# define _tenth		(*_tenth)
# define _tenthf	(*_tenthf)
# define _tenthl	(*_tenthl)
# define _sexag		(*_sexag)
# define _sexagf	(*_sexagf)
# define _sexagl	(*_sexagl)
# define _sqrt2		(*_sqrt2)
# define _sqrt2f	(*_sqrt2f)
# define _sqrt2l	(*_sqrt2l)
# define _sqrt1_2	(*_sqrt1_2)
# define _sqrt1_2f	(*_sqrt1_2f)
# define _sqrt1_2l	(*_sqrt1_2l)
# define _pi		(*_pi)
# define _pif		(*_pif)
# define _pil		(*_pil)
# define _degree	(*_degree)
# define _degreef	(*_degreef)
# define _degreel	(*_degreel)
# define _radian	(*_radian)
# define _radianf	(*_radianf)
# define _radianl	(*_radianl)
# define _gradl		(*_gradl)
# define _grad		(*_grad)
# define _gradf		(*_gradf)
# define _dargl		(*_dargl)
# define _darg		(*_darg)
# define _dargf		(*_dargf)
# define _twopi		(*_twopi)
# define _twopif	(*_twopif)
# define _twopil	(*_twopil)
# define _pi_2		(*_pi_2)
# define _pi_2f		(*_pi_2f)
# define _pi_2l		(*_pi_2l)
# define _pi_4		(*_pi_4)
# define _pi_4f		(*_pi_4f)
# define _pi_4l		(*_pi_4l)
# define _1_pi		(*_1_pi)
# define _1_pif		(*_1_pif)
# define _1_pil		(*_1_pil)
# define _2_pi		(*_2_pi)
# define _2_pif		(*_2_pif)
# define _2_pil		(*_2_pil)
# define _sqrtpi	(*_sqrtpi)
# define _sqrtpif	(*_sqrtpif)
# define _sqrtpil	(*_sqrtpil)
# define _1_sqrtpi	(*_1_sqrtpi)
# define _1_sqrtpif	(*_1_sqrtpif)
# define _1_sqrtpil	(*_1_sqrtpil)
# define _2_sqrtpi	(*_2_sqrtpi)
# define _2_sqrtpif	(*_2_sqrtpif)
# define _2_sqrtpil	(*_2_sqrtpil)
# define _e		(*_e)
# define _ef		(*_ef)
# define _el		(*_el)
# define _1_e		(*_1_e)
# define _1_ef		(*_1_ef)
# define _1_el		(*_1_el)
# define _ln10		(*_ln10)
# define _ln10f		(*_ln10f)
# define _ln10l		(*_ln10l)
# define _ln2		(*_ln2)
# define _ln2f		(*_ln2f)
# define _ln2l		(*_ln2l)
# define _lge		(*_lge)
# define _lgef		(*_lgef)
# define _lgel		(*_lgel)
# define _lg2		(*_lg2)
# define _lg2f		(*_lg2f)
# define _lg2l		(*_lg2l)
# define _log2e		(*_log2e)
# define _log2ef	(*_log2ef)
# define _log2el	(*_log2el)
# define _2log2e	(*_2log2e)
# define _2log2ef	(*_2log2ef)
# define _2log2el	(*_2log2el)
# define _log2e_2	(*_log2e_2)
# define _log2e_2f	(*_log2e_2f)
# define _log2e_2l	(*_log2e_2l)
# define _log2t		(*_log2t)
# define _log2tf	(*_log2tf)
# define _log2tl	(*_log2tl)
# define _euler		(*_euler)
# define _eulerf	(*_eulerf)
# define _eulerl	(*_eulerl)
# define epsilon	(*epsilon)
#endif

extern const float _ieeemaxf;
extern const double _ieeemax;
extern const long double _ieeemaxl;
extern const float _ieeeminf;
extern const double _ieeemin;
extern const long double _ieeeminl;
extern const float _infp;
extern const float _infn;
extern const float _qnan;
extern const float _snan;

extern const float _zero;		/* auch in FPU */
extern const float _one;		/* auch in FPU */
extern const float _two;
extern const float _three;
extern const float _four;
extern const float _five;
extern const float _six;
extern const float _seven;
extern const float _eight;
extern const float _nine;
extern const float _c10;
extern const float _c60;
extern const float _c90;
extern const float _c100;
extern const float _c160;
extern const float _c180;
extern const float _c360;
extern const float _half;
extern const float _oneahalf;
extern const float _twoahalf;
extern const float _threeahalf;
extern const double _third;
extern const float _thirdf;
extern const long double _thirdl;
extern const float _quart;
extern const double _tenth;
extern const float _tenthf;
extern const long double _tenthl;
extern const double _sexag;
extern const float _sexagf;
extern const long double _sexagl;

extern const double _sqrt2;
extern const float _sqrt2f;
extern const long double _sqrt2l;
extern const double _sqrt1_2;
extern const float _sqrt1_2f;
extern const long double _sqrt1_2l;

extern const double _pi;
extern const float _pif;
extern const long double _pil;	/* auch in FPU */
extern const double _degree;
extern const float _degreef;
extern const long double _degreel;
extern const double _radian;
extern const float _radianf;
extern const long double _radianl;
extern const long double _gradl;
extern const double _grad;
extern const float _gradf;
extern const long double _dargl;
extern const double _darg;
extern const float _dargf;
extern const double _twopi;
extern const float _twopif;
extern const long double _twopil;
extern const double _pi_2;
extern const float _pi_2f;
extern const long double _pi_2l;
extern const double _pi_4;
extern const float _pi_4f;
extern const long double _pi_4l;
extern const double _1_pi;
extern const float _1_pif;
extern const long double _1_pil;
extern const double _2_pi;
extern const float _2_pif;
extern const long double _2_pil;
extern const double _sqrtpi;
extern const float _sqrtpif;
extern const long double _sqrtpil;
extern const double _1_sqrtpi;
extern const float _1_sqrtpif;
extern const long double _1_sqrtpil;
extern const double _2_sqrtpi;
extern const float _2_sqrtpif;
extern const long double _2_sqrtpil;

extern const double _e;
extern const float _ef;
extern const long double _el;
extern const double _1_e;
extern const float _1_ef;
extern const long double _1_el;
extern const double _ln10;
extern const float _ln10f;
extern const long double _ln10l;
extern const double _ln2;
extern const float _ln2f;
extern const long double _ln2l;	/* auch in FPU */
extern const float _lge;
extern const double _lgef;
extern const long double _lgel;
extern const double _lg2;
extern const float _lg2f;
extern const long double _lg2l;	/* auch in FPU */
extern const double _log2e;
extern const float _log2ef;
extern const long double _log2el;	/* auch in FPU */
extern const double _2log2e;
extern const float _2log2ef;
extern const long double _2log2el;
extern const double _log2e_2;
extern const float _log2e_2f;
extern const long double _log2e_2l;
extern const double _log2t;
extern const float _log2tf;
extern const long double _log2tl;	/* auch in FPU */

extern const double _euler;
extern const float _eulerf;
extern const long double _eulerl;

extern double epsilon;

#define pi	_pi
#define pif	_pif
#define pil	_pil
#define degree	_degree
#define degreef	_degreef
#define degreel	_degreel
#define radian	_radian
#define radianf	_radianf
#define radianl	_radianl


#ifdef __GNUC__
char *lltobin(long long ll, char buf[65]);
#endif
long bswap(long x);
char *ctobin(char c, char buf[9]);
char *ltobin(long l, char buf[33]);
char *stobin(short s, char buf[17]);
char *fltoie(long double f, char ieee[21]);
long double ietofl(char ieee[21]);
long roman(char *string);
long rotate(long x, int count);
void *zeros(void *array, int size);

int desinit(int mode);
void desdone(void);
void dedes(unsigned char *block);
void endes(unsigned char *block);
void setkey(unsigned char *key);

float acosf(float x);
float acoshf(float x);
float acosh1pf(float x);
float acotf(float x);
float acot2f(float y, float x);
float acsf(float x);
float actf(float x);
float act2f(float y, float x);
pfMATRIX _add3f(pfMATRIX A, pfMATRIX B, pfMATRIX C);
pfVECTOR _addvecf(pfVECTOR A, pfVECTOR B, pfVECTOR c);
float angvecf(pfVECTOR a, pfVECTOR b);
float *arrsetf(float x, int n, float array[]);
float asectf(pfSPHERIC a, pfSPHERIC b);
float asinf(float x);
float asinhf(float x);
float asnf(float x);
float atanf(float x);
float atan2f(float y, float x);
float atnf(float x);
float atn2f(float y, float x);
float atanhf(float x);
float betragf(pfVECTOR v);
float bisectf(float from, float to, float (*func)(float));
pfCARTESIC cartf(pfSPHERIC s, pfCARTESIC c);
float cbevalf(float x, pfCHEBY c);
void cbeval2f(float x, pfCHEBY c, float pos[2]);
void cbeval3f(float x, pfCHEBY c, float pos[3]);
void cbinitf(pfCHEBY c);
void cbinit2f(pfCHEBY c);
void cbinit3f(pfCHEBY c);
float cbrtf(float x);
float ceilf(float x);
float combf(float x, float y);
float copysignf(float x, float y);
float copysign2f(float x, float y);
pfMATRIX copy3f(pfMATRIX A, pfMATRIX B);
float cosf(float x);
float coshf(float x);
float coshm1f(float x);
float cotf(float x);
pfVECTOR crossf(pfVECTOR a, pfVECTOR b, pfVECTOR c);
float csf(float x);
float ctf(float x);
float degf(float d, float m, float s);
float deg2f(float x);
float det3f(pfMATRIX mat);
float dgrf(float x);
float dgr2f(float x);
float dg2rdf(float x);
float dg2rd2f(float x);
pfSEXAGESI dmsf(float x, pfSEXAGESI s);
pANGLE dmssf(float x, int n, pANGLE a);
float dotf(pfVECTOR a, pfVECTOR b);
float dremf(float x, float y);
pfVECTOR dupvecf(pfVECTOR a, pfVECTOR b);
float erff(float x);
float erfcf(float x);
float expf(float x);
float expm1f(float x);
float fabsf(float x);
float factf(float x);
float floorf(float x);
float fmaxf(float x, float y);
float fminf(float x, float y);
float fmodf(float x, float y);
float fracf(float x);
float fremf(float x, float y);
float frexpf(float x, int *e);
float fshiftf(float x, int n);
float fsignf(float x);
void fswapf(float x[], float y[], int n);
void fswap2f(float *x, float *y);
float fyl2xf(float x, float y);
float fyl2xp1f(float x, float y);
float f2xm1f(float x);
float gammaf(float x);
float gcmf(float x, float y);
float hacf(float x);
float hatf(float x);
float havf(float x);
float hcsf(float x);
float hsnf(float x);
float htnf(float x);
float hypotf(float x, float y);
int iceilf(float x);
int ifloorf(float x);
float ip3_extrf(float tab[3]);
float ip3_funcf(float x, float tab[3]);
float ip3_rootf(float y, float tab[3]);
float ip4_extrf(float tab[4]);
float ip4_funcf(float x, float tab[4]);
float ip4_rootf(float y, float tab[4]);
float ip5_extrf(float tab[5]);
float ip5_funcf(float x, float tab[5]);
float ip5_rootf(float y, float tab[5]);
int irintf(float x);
pfVECTOR _irotate3f(pfVECTOR E, pcfMATRIX A, pfVECTOR R);
pfCARTESIC _irot_xf(pfCARTESIC rec, pcfSINCOS a, pfCARTESIC rot);
pfCARTESIC _irot_yf(pfCARTESIC rec, pcfSINCOS a, pfCARTESIC rot);
pfCARTESIC _irot_zf(pfCARTESIC rec, pcfSINCOS a, pfCARTESIC rot);
int itruncf(float x);
float j0f(float x);
float j1f(float x);
float jnf(int n, float x);
float lagrangef(float x, int n, pcfPAIR const dat);
float lcmf(float x, float y);
float ldexpf(float x, int y);
float lgammaf(float x, int *s);
float logf(float x);
float logbf(float x);
float log1pf(float x);
float log2f(float x);
float log2p1f(float x);
float log10f(float x);
float log10p1f(float x);
float logyf(float x, float y);
float logy1pf(float x, float y);
float magnitf(float x, float y, float z);
float modff(float x, float *y);
void multfitf(pfUMATRIX , pfUVECTOR V);
pfUMATRIX mxaddf(pfUMATRIX A, pfUMATRIX B);
pfUMATRIX mxdupf(pfUMATRIX A, pfUMATRIX B);
pfUMATRIX mxmulf(pfUMATRIX A, pfUMATRIX B, pfUMATRIX C);
pfUMATRIX mxscalef(float k, pfUMATRIX A);
pfUVECTOR mxsolvef(pfUMATRIX , pfUVECTOR V);
pfUMATRIX mxsubf(pfUMATRIX A, pfUMATRIX B);
pfUMATRIX mxtransf(pfUMATRIX A, pfUMATRIX B);
float ndtrf(float x);
float ndtr1mf(float x);
float newtonf(float start, float (*func)(float));
float permf(float x, float y);
pfPOLAR plrf(pfRECTAN r, pfPOLAR p);
pfPOLAR polf(pfRECTAN r, pfPOLAR p);
float _polyf(float x, unsigned n, const float *coeff);
float _polydgf(float x, unsigned n, const float *coeff);
float _polydg2rdf(float x, unsigned n, const float *coeff);
void polyfitf(pfUMATRIX , pfUPOLYNOM P);
float _polyrdf(float x, unsigned n, const float *coeff);
float powf(float x, float y);
float powm1f(float x, float y);
float pow2f(float x);
float pow2m1f(float x);
float pow10f(float x);
float pow10m1f(float x);
int quadf(pfQUAD q, pfPAIR x);
float radf(float x);
float rad2f(float x);
float rd2dgf(float x);
float rd2dg2f(float x);
pfRECTAN rctf(pfPOLAR p, pfRECTAN r);
pfRECTAN recf(pfPOLAR p, pfRECTAN r);
void regexpf(int n, pcfPAIR const dat, pfREGANALYSIS stat);
void reglinf(int n, pcfPAIR const dat, pfREGANALYSIS stat);
void reglogf(int n, pcfPAIR const dat, pfREGANALYSIS stat);
void regpowf(int n, pcfPAIR const dat, pfREGANALYSIS stat);
float rintf(float x);
float rndf(float x);
float rndexpf(float x);
float rndnormf(float x, float s);
float rndseedf(float x);
pfVECTOR _rotate3f(pfVECTOR E, pcfMATRIX A, pfVECTOR R);
pfCARTESIC _rot_xf(pfCARTESIC rec, pcfSINCOS a, pfCARTESIC rot);
pfCARTESIC _rot_yf(pfCARTESIC rec, pcfSINCOS a, pfCARTESIC rot);
pfCARTESIC _rot_zf(pfCARTESIC rec, pcfSINCOS a, pfCARTESIC rot);
float roundf(float x, int n);
pfVECTOR _scalarf(float a, pfVECTOR u, pfVECTOR v);
pfMATRIX _scale3f(float k, pfMATRIX A, pfMATRIX B);
float sinf(float x);
float sinhf(float x);
float sincosf(float x, float *c);
void sincos2f(float x, pfSINCOS a);
float snf(float x);
pfSPHERIC sphericf(fCARTESIC c, fSPHERIC s);
pfVECTOR _solve3f(pfVECTOR B, pfMATRIX A, pfVECTOR R);
float sqrtf(float x);
void statisf(int n, const float *const dat, pfREGANALYSIS stat);
pfMATRIX _sub3f(pfMATRIX A, pfMATRIX B, pfMATRIX C);
pfVECTOR _subvecf(pfVECTOR A, pfVECTOR B, pfVECTOR c);
float tanf(float x);
float tanhf(float x);
pfMATRIX _transpose3f(pfMATRIX A, pfMATRIX B);
float truncf(float x);
float tnf(float x);
pfVECTOR univecf(pfVECTOR a, pfVECTOR u);
void vecxmatf(pfVECTOR v, pfMATRIX m);
float y0f(float x);
float y1f(float x);
float ynf(int n, float x);

double acos(double x);
double acosh(double x);
double acosh1p(double x);
double acot(double x);
double acot2(double y, double x);
double acs(double x);
double act(double x);
double act2(double y, double x);
pMATRIX _add3(pMATRIX A, pMATRIX B, pMATRIX C);
pVECTOR _addvec(pVECTOR A, pVECTOR B, pVECTOR c);
double angvec(pVECTOR a, pVECTOR b);
double *arrset(double x, int n, double array[]);
double asect(pSPHERIC a, pSPHERIC b);
double asin(double x);
double asinh(double x);
double asn(double x);
double atan(double x);
double atan2(double y, double x);
double atn(double x);
double atn2(double y, double x);
double atanh(double x);
double betrag(pVECTOR v);
double bisect(double from, double to, double (*func)(double));
pCARTESIC cart(pSPHERIC s, pCARTESIC c);
double cbeval(double x, const pCHEBY c);
void cbeval2(double x, const pCHEBY c, double pos[2]);
void cbeval3(double x, const pCHEBY c, double pos[3]);
void cbinit(pCHEBY c);
void cbinit2(pCHEBY c);
void cbinit3(pCHEBY c);
double cbrt(double x);
double ceil(double x);
double comb(double x, double y);
double copysign(double x, double y);
double copysign2(double x, double y);
pMATRIX copy3(pMATRIX A, pMATRIX B);
double cos(double x);
double cosh(double x);
double coshm1(double x);
double cot(double x);
pVECTOR cross(pVECTOR a, pVECTOR b, pVECTOR c);
double cs(double x);
double ct(double x);
double deg(double d, double m, double s);
double deg2(double x);
double det3(pMATRIX mat);
double dgr(double x);
double dgr2(double x);
double dg2rd(double x);
double dg2rd2(double x);
pSEXAGESI dms(double x, pSEXAGESI s);
pANGLE dmss(double x, int n, pANGLE a);
double dot(pVECTOR a, pVECTOR b);
double drem(double x, double y);
pVECTOR dupvec(pVECTOR a, pVECTOR b);
double erf(double x);
double erfc(double x);
double exp(double x);
double expm1(double x);
double fabs(double x);
double fact(double x);
double floor(double x);
double fmax(double x, double y);
double fmin(double x, double y);
double fmod(double x, double y);
double frac(double x);
double frem(double x, double y);
double frexp(double x, int *e);
double fshift(double x, int n);
double fsign(double x);
void fswap(double x[], double y[], int n);
void fswap2(double *x, double *y);
double fyl2x(double x, double y);
double fyl2xp1(double x, double y);
double f2xm1(double x);
double gamma(double x);
double gcm(double x, double y);
double hac(double x);
double hat(double x);
double hav(double x);
double hcs(double x);
double hsn(double x);
double htn(double x);
double hypot(double x, double y);
int iceil(double x);
int ifloor(double x);
double ip3_extr(double tab[3]);
double ip3_func(double x, double tab[3]);
double ip3_root(double y, double tab[3]);
double ip4_extr(double tab[4]);
double ip4_func(double x, double tab[4]);
double ip4_root(double y, double tab[4]);
double ip5_extr(double tab[5]);
double ip5_func(double x, double tab[5]);
double ip5_root(double y, double tab[5]);
int irint(double x);
pVECTOR _irotate3(pVECTOR E, pcMATRIX A, pVECTOR R);
pCARTESIC _irot_x(pCARTESIC rec, pcSINCOS a, pCARTESIC rot);
pCARTESIC _irot_y(pCARTESIC rec, pcSINCOS a, pCARTESIC rot);
pCARTESIC _irot_z(pCARTESIC rec, pcSINCOS a, pCARTESIC rot);
int itrunc(double x);
double j0(double x);
double j1(double x);
double jn(int n, double x);
double lagrange(double x, int n, pcPAIR const dat);
double lcm(double x, double y);
double ldexp(double x, int y);
double lgamma(double x, int *s);
double log(double x);
double logb(double x);
double log1p(double x);
double log2(double x);
double log2p1(double x);
double log10(double x);
double log10p1(double x);
double logy(double x, double y);
double logy1p(double x, double y);
double magnit(double x, double y, double z);
double modf(double x, double *y);
void multfit(pUMATRIX , pUVECTOR V);
pUMATRIX mxadd(pUMATRIX A, pUMATRIX B);
pUMATRIX mxdup(pUMATRIX A, pUMATRIX B);
pUMATRIX mxmul(pUMATRIX A, pUMATRIX B, pUMATRIX C);
pUMATRIX mxscale(double k, pUMATRIX A);
pUVECTOR mxsolve(pUMATRIX , pUVECTOR V);
pUMATRIX mxsub(pUMATRIX A, pUMATRIX B);
pUMATRIX mxtrans(pUMATRIX A, pUMATRIX B);
double ndtr(double x);
double ndtr1m(double x);
double newton(double start, double (*func)(double));
double perm(double x, double y);
pPOLAR plr(pRECTAN r, pPOLAR p);
pPOLAR pol(pRECTAN r, pPOLAR p);
double _poly(double x, unsigned n, const double *coeff);
double _polydg(double x, unsigned n, const double *coeff);
double _polydg2rd(double x, unsigned n, const double *coeff);
void polyfit(pUMATRIX , pUPOLYNOM P);
double _polyrd(double x, unsigned n, const double *coeff);
double pow(double x, double y);
double powm1(double x, double y);
double pow2(double x);
double pow2m1(double x);
double pow10(double x);
double pow10m1(double x);
int quad(pQUAD q, pPAIR x);
double rad(double x);
double rad2(double x);
double rd2dg(double x);
double rd2dg2(double x);
pRECTAN rct(pPOLAR p, pRECTAN r);
pRECTAN rec(pPOLAR p, pRECTAN r);
void regexp(int n, pcPAIR const dat, pREGANALYSIS stat);
void reglin(int n, pcPAIR const dat, pREGANALYSIS stat);
void reglog(int n, pcPAIR const dat, pREGANALYSIS stat);
void regpow(int n, pcPAIR const dat, pREGANALYSIS stat);
double rint(double x);
double rnd(double x);
void *rnd2(void);
double rndexp(double x);
double rndnorm(double x, double s);
double rndseed(double x);
void *rnd2seed(unsigned char *x);
pVECTOR _rotate3(pVECTOR E, pcMATRIX A, pVECTOR R);
pCARTESIC _rot_x(pCARTESIC rec, pcSINCOS a, pCARTESIC rot);
pCARTESIC _rot_y(pCARTESIC rec, pcSINCOS a, pCARTESIC rot);
pCARTESIC _rot_z(pCARTESIC rec, pcSINCOS a, pCARTESIC rot);
double round(double x, int n);
pVECTOR _scalar(double a, pVECTOR u, pVECTOR v);
pMATRIX _scale3(double k, pMATRIX A, pMATRIX B);
double sin(double x);
double sinh(double x);
double sincos(double x, double *c);
void sincos2(double x, pSINCOS a);
double sn(double x);
pSPHERIC spheric(pCARTESIC c, pSPHERIC s);
pVECTOR _solve3(pVECTOR B, pMATRIX A, pVECTOR R);
double sqrt(double x);
void statis(int n, const double *const dat, pREGANALYSIS stat);
pMATRIX _sub3(pMATRIX A, pMATRIX B, pMATRIX C);
pVECTOR _subvec(pVECTOR A, pVECTOR B, pVECTOR c);
double tan(double x);
double tanh(double x);
pMATRIX _transpose3(pMATRIX A, pMATRIX B);
double trunc(double x);
double tn(double x);
pVECTOR univec(pVECTOR A, pVECTOR U);
void vecxmat(pVECTOR A, pMATRIX );
double y0(double x);
double y1(double x);
double yn(int n, double x);

long double acosl(long double x);
long double acoshl(long double x);
long double acosh1pl(long double x);
long double acotl(long double x);
long double acot2l(long double y, long double x);
long double acsl(long double x);
long double actl(long double x);
long double act2l(long double y, long double x);
pldMATRIX _add3l(pldMATRIX A, pldMATRIX B, pldMATRIX C);
pldVECTOR _addvecl(pldVECTOR A, pldVECTOR B, pldVECTOR c);
long double angvecl(pldVECTOR a, pldVECTOR b);
long double *arrsetl(long double x, int n, long double array[]);
long double asectl(pldSPHERIC a, pldSPHERIC b);
long double asinl(long double x);
long double asinhl(long double x);
long double asnl(long double x);
long double atanl(long double x);
long double atan2l(long double y, long double x);
long double atnl(long double x);
long double atn2l(long double y, long double x);
long double atanhl(long double x);
long double betragl(pldVECTOR v);
long double bisectl(long double from, long double to, long double (*func)(long double));
pldCARTESIC cartl(pldSPHERIC s, pldCARTESIC c);
long double cbevall(long double x, pldCHEBY c);
void cbeval2l(long double x, pldCHEBY c, long double pos[2]);
void cbeval3l(long double x, pldCHEBY c, long double pos[3]);
void cbinitl(pldCHEBY c);
void cbinit2l(pldCHEBY c);
void cbinit3l(pldCHEBY c);
long double cbrtl(long double x);
long double ceill(long double x);
long double combl(long double x, long double y);
long double copysignl(long double x, long double y);
long double copysign2l(long double x, long double y);
pldMATRIX copy3l(pldMATRIX A, pldMATRIX B);
long double cosl(long double x);
long double coshl(long double x);
long double coshm1l(long double x);
long double cotl(long double x);
pldVECTOR crossl(pldVECTOR a, pldVECTOR b, pldVECTOR c);
long double csl(long double x);
long double ctl(long double x);
long double degl(long double d, long double m, long double s);
long double deg2l(long double x);
long double det3l(pldMATRIX mat);
long double dgrl(long double x);
long double dgr2l(long double x);
long double dg2rdl(long double x);
long double dg2rd2l(long double x);
pldSEXAGESI dmsl(long double x, pldSEXAGESI s);
pANGLE dmssl(long double x, int n, pANGLE a);
long double dotl(pldVECTOR a, pldVECTOR b);
long double dreml(long double x, long double y);
pldVECTOR dupvecl(pldVECTOR a, pldVECTOR b);
long double erfl(long double x);
long double erfcl(long double x);
long double expl(long double x);
long double expm1l(long double x);
long double fabsl(long double x);
long double factl(long double x);
long double floorl(long double x);
long double fmaxl(long double x, long double y);
long double fminl(long double x, long double y);
long double fmodl(long double x, long double y);
long double fracl(long double x);
long double freml(long double x, long double y);
long double frexpl(long double x, int *e);
long double fshiftl(long double x, int n);
long double fsignl(long double x);
void fswapl(long double x[], long double y[], int n);
void fswap2l(long double *x, long double *y);
long double fyl2xl(long double x, long double y);
long double fyl2xp1l(long double x, long double y);
long double f2xm1l(long double x);
long double gammal(long double x);
long double gcml(long double x, long double y);
long double hacl(long double x);
long double hatl(long double x);
long double havl(long double x);
long double hcsl(long double x);
long double hsnl(long double x);
long double htnl(long double x);
long double hypotl(long double x, long double y);
int iceill(long double x);
int ifloorl(long double x);
long double ip3_extrl(long double tab[3]);
long double ip3_funcl(long double x, long double tab[3]);
long double ip3_rootl(long double y, long double tab[3]);
long double ip4_extrl(long double tab[4]);
long double ip4_funcl(long double x, long double tab[4]);
long double ip4_rootl(long double y, long double tab[4]);
long double ip5_extrl(long double tab[5]);
long double ip5_funcl(long double x, long double tab[5]);
long double ip5_rootl(long double y, long double tab[5]);
int irintl(long double x);
pldVECTOR _irotate3l(pldVECTOR E, pcldMATRIX A, pldVECTOR R);
pldCARTESIC _irot_xl(pldCARTESIC rec, pcldSINCOS a, pldCARTESIC rot);
pldCARTESIC _irot_yl(pldCARTESIC rec, pcldSINCOS a, pldCARTESIC rot);
pldCARTESIC _irot_zl(pldCARTESIC rec, pcldSINCOS a, pldCARTESIC rot);
int itruncl(long double x);
long double j0l(long double x);
long double j1l(long double x);
long double jnl(int n, long double x);
long double lagrangl(long double x, int n, pcldPAIR const dat);
long double lcml(long double x, long double y);
long double ldexpl(long double x, int y);
long double lgammal(long double x, int *s);
long double logl(long double x);
long double logbl(long double x);
long double log1pl(long double x);
long double log2l(long double x);
long double log2p1l(long double x);
long double log10l(long double x);
long double log10p1l(long double x);
long double logyl(long double x, long double y);
long double logy1pl(long double x, long double y);
long double magnitl(long double x, long double y, long double z);
long double modfl(long double x, long double *y);
void multfitl(pldUMATRIX , pldUVECTOR V);
pldUMATRIX mxaddl(pldUMATRIX A, pldUMATRIX B);
pldUMATRIX mxdupl(pldUMATRIX A, pldUMATRIX B);
pldUMATRIX mxmull(pldUMATRIX A, pldUMATRIX B, pldUMATRIX C);
pldUMATRIX mxscalel(long double k, pldUMATRIX A);
pldUVECTOR mxsolvel(pldUMATRIX , pldUVECTOR V);
pldUMATRIX mxsubl(pldUMATRIX A, pldUMATRIX B);
pldUMATRIX mxtransl(pldUMATRIX A, pldUMATRIX B);
long double ndtrl(long double x);
long double ndtr1ml(long double x);
long double newtonl(long double start, long double (*func)(long double));
long double perml(long double x, long double y);
pldPOLAR plrl(pldRECTAN r, pldPOLAR p);
pldPOLAR poll(pldRECTAN r, pldPOLAR p);
long double _polyl(long double x, unsigned n, const long double *coeff);
long double _polydgl(long double x, unsigned n, const long double *coeff);
long double _polydg2rdl(long double x, unsigned n, const long double *coeff);
void polyfitl(pldUMATRIX , pldUPOLYNOM P);
long double _polyrdl(long double x, unsigned n, const long double *coeff);
long double powl(long double x, long double y);
long double powm1l(long double x, long double y);
long double pow2l(long double x);
long double pow2m1l(long double x);
long double pow10l(long double x);
long double pow10m1l(long double x);
int quadl(pldQUAD q, pldPAIR x);
long double radl(long double x);
long double rad2l(long double x);
long double rd2dgl(long double x);
long double rd2dg2l(long double x);
pldRECTAN rctl(pldPOLAR p, pldRECTAN r);
pldRECTAN recl(pldPOLAR p, pldRECTAN r);
void regexpl(int n, pcldPAIR const dat, pldREGANALYSIS stat);
void reglinl(int n, pcldPAIR const dat, pldREGANALYSIS stat);
void reglogl(int n, pcldPAIR const dat, pldREGANALYSIS stat);
void regpowl(int n, pcldPAIR const dat, pldREGANALYSIS stat);
long double rintl(long double x);
long double rndl(long double x);
long double rndexpl(long double x);
long double rndnorml(long double x, long double s);
long double rndseedl(long double x);
pldVECTOR _rotate3l(pldVECTOR E, pcldMATRIX A, pldVECTOR R);
pldCARTESIC _rot_xl(pldCARTESIC rec, pcldSINCOS a, pldCARTESIC rot);
pldCARTESIC _rot_yl(pldCARTESIC rec, pcldSINCOS a, pldCARTESIC rot);
pldCARTESIC _rot_zl(pldCARTESIC rec, pcldSINCOS a, pldCARTESIC rot);
long double roundl(long double x, int n);
pldVECTOR _scalarl(long double a, pldVECTOR u, pldVECTOR v);
pldMATRIX _scale3l(long double k, pldMATRIX A, pldMATRIX B);
long double sinl(long double x);
long double sinhl(long double x);
long double sincosl(long double x, long double *c);
void sincos2l(long double x, pldSINCOS a);
long double snl(long double x);
pldVECTOR _solve3l(pldVECTOR B, pldMATRIX A, pldVECTOR R);
pldSPHERIC sphericl(pldCARTESIC c, pldSPHERIC s);
long double sqrtl(long double x);
void statisl(int n, const long double *const dat, pldREGANALYSIS stat);
pldMATRIX _sub3l(pldMATRIX A, pldMATRIX B, pldMATRIX C);
pldVECTOR _subvecl(pldVECTOR A, pldVECTOR B, pldVECTOR c);
long double tanl(long double x);
long double tanhl(long double x);
pldMATRIX _transpose3l(pldMATRIX A,pldMATRIX B);
long double truncl(long double x);
long double tnl(long double x);
pldVECTOR univecl(pldVECTOR a, pldVECTOR u);
void vecxmatl(pldVECTOR V, pldMATRIX );
long double y0l(long double x);
long double y1l(long double x);
long double ynl(int n, long double x);



#define	scalb(x, i)	ldexp(x,i)		/* BSD */
#define	scalbf(x, i)	ldexpf(x,i)		/* BSD */
#define	scalbl(x, i)	ldexpl(x,i)		/* BSD */

#define add3f(a,b)		_add3f(a,b,a)
#define addvecf(a,b)		_addvecf(a,b,a)
#define irotate3f(a,b)		_irotate3f(a,b,a)
#define irot_xf(a,b)		_irot_xf(a,b,a)
#define irot_yf(a,b)		_irot_yf(a,b,a)
#define irot_zf(a,b)		_irot_zf(a,b,a)
#define rotate3f(a,b)		_rotate3f(a,b,a)
#define rot_xf(a,b)		_rot_xf(a,b,a)
#define rot_yf(a,b)		_rot_yf(a,b,a)
#define rot_zf(a,b)		_rot_zf(a,b,a)
#define scalarf(a,b)		_scalarf(a,b,b)
#define scale3f(a,b)		_scale3f(a,b,a)
#define solve3f(a,b)		_solve3f(a,b,a)
#define sub3f(a,b)		_sub3f(a,b,a)
#define subvecf(a,b)		_subvecf(a,b,a)
#define transpose3f(a)		_transpose3f(a,a)

#define add3(a,b)		_add3(a,b,a)
#define addvec(a,b)		_addvec(a,b,a)
#define irot_x(a,b)		_irot_x(a,b,a)
#define irot_y(a,b)		_irot_y(a,b,a)
#define irot_z(a,b)		_irot_z(a,b,a)
#define irotate3(a,b)		_irotate3(a,b,a)
#define rot_x(a,b)		_rot_x(a,b,a)
#define rot_y(a,b)		_rot_y(a,b,a)
#define rot_z(a,b)		_rot_z(a,b,a)
#define rotate3(a,b)		_rotate3(a,b,a)
#define scalar(a,b)		_scalar(a,b,b)
#define scale3(a,b)		_scale3(a,b,a)
#define solve3(a,b)		_solve3(a,b,a)
#define sub3(a,b)		_sub3(a,b,a)
#define subvec(a,b)		_subvec(a,b,a)
#define transpose3(a)		_transpose3(a,a)

#define add3l(a,b)		_add3l(a,b,a)
#define addvecl(a,b)		_addvecl(a,b,a)
#define irotate3l(a,b)		_irotate3l(a,b,a)
#define irot_xl(a,b)		_irot_xl(a,b,a)
#define irot_yl(a,b)		_irot_yl(a,b,a)
#define irot_zl(a,b)		_irot_zl(a,b,a)
#define rotate3l(a,b)		_rotate3l(a,b,a)
#define rot_xl(a,b)		_rot_xl(a,b,a)
#define rot_yl(a,b)		_rot_yl(a,b,a)
#define rot_zl(a,b)		_rot_zl(a,b,a)
#define scalarl(a,b)		_scalarl(a,b,b)
#define scale3l(a,b)		_scale3l(a,b,a)
#define solve3l(a,b)		_solve3l(a,b,a)
#define sub3l(a,b)		_sub3l(a,b,a)
#define subvecl(a,b)		_subvecl(a,b,a)
#define transpose3l(a)		_transpose3l(a,a)


#define TRM(p)			(sizeof(p)/sizeof(p[0]))
#define PWR(p)			(sizeof(p)/sizeof(p[0])-1)
#define poly(t,p)		_poly(t,PWR(p),p)
#define polydg(t,p)		_polydg(t,PWR(p),p)
#define polydg2rd(t,p)		_polydg2rd(t,PWR(p),p)
#define polyrd(t,p)		_polyrd(t,PWR(p),p)


#endif	/* __ASSEMBLER__ */


#if defined (__cplusplus)
}
#endif


#endif		/* FMATH_H */

