/* $Id$ */
/*------------------------------------------------------*/
/*     Name:              fmathctl.h                    */
/*------------------------------------------------------*/
/*  Aufgabe: Koprozessorsteuerung, Rechengang-Kontrolle */
/*------------------------------------------------------*/
/*   Copyright (C) 1995 - 2000 by Michael Schroeder     */
/*------------------------------------------------------*/


/*
 * LIBFMATH is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 */


#ifndef FMATHCTL_H
# define FMATHCTL_H

#if defined (__cplusplus)
extern "C" {
#endif


#ifndef mcode
# define mcode	__asm__ __volatile__
#endif

/*
 * Nummernschema: MSB = Gruppennummer, LSB = Funktionsnummer (alphabetisch)
 * Identifikationsnummern der Funktionen "ALIGN 2":
 * 	`long double' = `double+1'	-> Bit 0 gesetzt
 *	`float'       = `double+2'	-> Bit 1 gesetzt
 *	`reserved'    = `double+3'
 */
#ifdef LDBL
# define FUNCID(x)	(ID_##x + 1)
#elif FLT
# define FUNCID(x)	(ID_##x + 2)
#else
# define FUNCID(x)	(ID_##x)
#endif

#define ID_EMPTY	0x0000		/* Systembereich		*/

/* Low-Level Funktionen */

#define ID_FXAM		0x0100		/* Zahlenuntersuchungen		*/
#define ID_ISSIGN	0x0104
#define ID_ISNORM	0x0110
#define ID_ISFINIT	0x0114
#define ID_ISINF	0x0118
#define ID_ISNAN	0x011c
#define ID_ISODD	0x0130

#define ID_FABS		0x0200		/* Zahlenzerlegungen		*/
#define ID_FSIGN	0x0204
#define ID_FRAC		0x0208
#define ID_MODF		0x020c
#define ID_FREXP	0x0210

#define ID_CEIL		0x0300		/* Rundungsfunktionen		*/
#define ID_FLOOR	0x0304
#define ID_RINT		0x0308
#define ID_TRUNC	0x030c
#define ID_ICEIL	0x0310
#define ID_IFLOOR	0x0314
#define ID_IRINT	0x0318
#define ID_ITRUNC	0x031c
#define ID_ROUND	0x0320

#define ID_FMOD		0x0500		/* Zahlen(Winkel)reduktionen	*/
#define ID_DREM		0x0504
#define ID_FREM		0x0508
#define ID_RAD		0x0510
#define ID_RAD2		0x0514
#define ID_DGR		0x0520
#define ID_DGR2		0x0524
#define ID_RD2DG	0x0530
#define ID_RD2DG2	0x0534
#define ID_DG2RD	0x0540
#define ID_DG2RD2	0x0544

#define ID_COPYSIGN	0x0a00		/* Zahlenmanipulation		*/
#define ID_COPYSIGN2	0x0a04
#define ID_FSHIFT	0x0a10
#define ID_FSWAP	0x0a20
#define ID_FMAX		0x0a30
#define ID_FMIN		0x0a34

/* High-Level Funktionen */

#define ID_SIN		0x1000		/* Trigonometrische Funktionen	*/
#define ID_COS		0x1004
#define ID_TAN		0x1008
#define ID_COT		0x100c
#define ID_SINCOS	0x1010
#define ID_SINCOS2	0x1014
#define ID_SN		0x1020
#define ID_CS		0x1024
#define ID_TN		0x1028
#define ID_CT		0x102c
#define ID_HAV		0x1030
#define ID_HAC		0x1034
#define ID_HAT		0x1038
#define ID_HSN		0x1040
#define ID_HCS		0x1044
#define ID_HTN		0x1048

#define ID_ASIN		0x1100		/* Inverse trigon. Funktionen	*/
#define ID_ACOS		0x1104
#define ID_ATAN		0x1108
#define ID_ACOT		0x110c
#define ID_ATAN2	0x1110
#define ID_ACOT2	0x1114
#define ID_ASN		0x1120
#define ID_ACS		0x1124
#define ID_ATN		0x1128
#define ID_ACT		0x112c
#define ID_ATN2		0x1130
#define ID_ACT2		0x1134
#define ID_ASECT	0x1150

#define ID_SINH		0x1200		/* Hyperbolische Funktionen	*/
#define ID_COSH		0x1204
#define ID_TANH		0x1208
#define ID_COSHM1	0x1210

#define ID_ASINH	0x1300		/* Inverse hyperbolische Funkt.	*/
#define ID_ACOSH	0x1304
#define ID_ATANH	0x1308
#define ID_ACOSH1P	0x1310

#define ID_LOG		0x1400		/* Logarithmische Funktionen	*/
#define ID_LOG10	0x1404
#define ID_LOG2		0x1408
#define ID_LOGY		0x140c
#define ID_LOG1P	0x1410
#define ID_LOG10P1	0x1414
#define ID_LOG2P1	0x1418
#define ID_LOGY1P	0x141c
#define ID_LOGB		0x1420
#define ID_FYL2X	0x14f0
#define ID_FYL2XP1	0x14f4

#define	ID_SQRT		0x1500		/* Exponentiale Funktionen	*/
#define ID_CBRT		0x1504
#define ID_HYPOT	0x1510
#define ID_MAGNIT	0x1514
#define ID_LDEXP	0x1520
#define ID_EXP		0x1530
#define ID_POW10	0x1534
#define ID_POW2		0x1538
#define ID_POW		0x1540
#define ID_EXPM1	0x1550
#define ID_POW10M1	0x1554
#define ID_POW2M1	0x1558
#define ID_POWM1	0x1560
#define ID_F2XM1	0x15f0

#define ID_FLTOIE	0x1a00		/* Zahlentransformationen	*/
#define ID_IETOFL	0x1a04
#define ID_REC		0x1a10
#define ID_POL		0x1a14
#define ID_RCT		0x1a18
#define ID_PLR		0x1a1c
#define ID_CART		0x1a20
#define ID_SPHERIC	0x1a24
#define ID_DEG		0x1a30
#define ID_DEG2		0x1a34
#define ID_DMS		0x1a40
#define ID_DMSS		0x1a44

#define ID_FACT		0x2000		/* Kombinatorik	*/
#define ID_GAMMA	0x2004
#define ID_LGAMMA	0x2008
#define ID_COMB		0x2010
#define ID_PERM		0x2014
#define ID_GCM		0x2020
#define ID_LCM		0x2024

#define ID_ERF		0x2100		/* Statistik */
#define ID_ERFC		0x2104
#define ID_NDTR		0x2110
#define ID_NDTR1M	0x2114
#define ID_RNDSEED	0x2120
#define ID_RND		0x2124
#define ID_RNDEXP	0x2128
#define ID_RNDNORM	0x212c
#define ID_STATIS	0x2130
#define ID_REGLIN	0x2140
#define ID_REGLOG	0x2144
#define ID_REGEXP	0x2148
#define ID_REGPOW	0x214c

#define ID_J0		0x2200		/* Analysis */
#define ID_Y0		0x2204
#define ID_J1		0x2208
#define ID_Y1		0x220c
#define ID_JN		0x2210
#define ID_YN		0x2214
#define ID_BISECT	0x2220
#define ID_NEWTON	0x2224

#define	ID_BETRAG	0x2400		/* Vektoren */
#define ID_SCALAR	0x2404
#define	ID_DOT		0x2408
#define	ID_CROSS	0x240c
#define	ID_DUPVEC	0x2410
#define	ID_UNIVEC	0x2414
#define ID_ADDVEC	0x2420
#define ID_SUBVEC	0x2424
#define	ID_ANGVEC	0x2430
#define ID_ROTX		0x2440
#define ID_ROTY		0x2444
#define ID_ROTZ		0x2448
#define ID_IROTX	0x2450
#define ID_IROTY	0x2454
#define ID_IROTZ	0x2458

#define ID_LAGRANGE	0x2550		/* Interpolation */
#define ID_IP3FUNC	0x2550
#define ID_IP3ROOT	0x2554
#define ID_IP3EXTR	0x2558
#define ID_IP4FUNC	0x2560
#define ID_IP4ROOT	0x2564
#define ID_IP4EXTR	0x2568
#define ID_IP5FUNC	0x2570
#define ID_IP5ROOT	0x2574
#define ID_IP5EXTR	0x2578

#define ID_CBINIT	0x2600		/* Approximation */
#define ID_CBINIT2	0x2604
#define ID_CBINIT3	0x2608
#define ID_CBEVAL	0x2610
#define ID_CBEVAL2	0x2614
#define ID_CBEVAL3	0x2618
#define ID_MULTFIT	0x2650
#define ID_POLYFIT	0x2654

#define ID_POLY		0x2800		/* Polynome */
#define ID_POLYRD	0x2804
#define ID_POLYDG	0x2808
#define ID_POLYDG2RD	0x280c
#define ID_POLYMUL	0x2810
#define ID_QUAD		0x2850

#define ID_COPY3	0x2a00		/* 3x3-Matrizen */
#define ID_TRANSPOSE3	0x2a10
#define ID_SCALE3	0x2a14
#define ID_DET3		0x2a18
#define ID_ADD3		0x2a20
#define ID_SUB3		0x2a24
#define ID_ROTATE3	0x2a40
#define ID_IROTATE3	0x2a44
#define ID_SOLVE3	0x2a50

#define ID_MXDUP	0x2b00		/* Unbestimmte Matrizen	*/
#define ID_MXTRANS	0x2b10
#define ID_MXSCALE	0x2b14
#define ID_MXADD	0x2b20
#define ID_MXSUB	0x2b24
#define ID_MXMUL	0x2b30
#define ID_MXSOLVE	0x2b50


#define ID_XABS		0x3000		/* Komplexe Funktionen */
#define ID_XARG		0x3004
#define ID_XSQRT	0x3008
#define ID_XSIN		0x3010
#define ID_XCOS		0x3014
#define ID_XTAN		0x3018
#define ID_XASIN	0x3020
#define ID_XACOS	0x3024
#define ID_XATAN	0x3028
#define ID_XSINH	0x3030
#define ID_XCOSH	0x3034
#define ID_XTANH	0x3038
#define ID_XASINH	0x3040
#define ID_XACOSH	0x3044
#define ID_XATANH	0x3048
#define ID_XLOG		0x3050
#define ID_XLOG10	0x3054
#define ID_XLOG2	0x3058
#define ID_XLOGY	0x305c
#define ID_XEXP		0x3060
#define ID_XPOW		0x3064
#define ID_XPOW10	0x3068
#define ID_XPOW2	0x306c


#ifndef MCW_EM		/* Control Word (wie in FLOAT.H) */

# define MCW_EM			0x003f	/* Ausnahmebedingungen		*/
# define EM_INVALID		0x0001	/* Ungueltige Operation		*/
# define EM_DENORMAL		0x0002	/* Nicht normalisierte Zahl	*/
# define EM_ZERODIVIDE		0x0004	/* Dividieren durch 0		*/
# define EM_OVERFLOW		0x0008	/* Zahl zu gross			*/
# define EM_UNDERFLOW		0x0010	/* Zahl zu klein		*/
# define EM_INEXACT		0x0020	/* Rundungsfehler		*/

# define MCW_PC			0x0300	/* Genauigkeitseinstellung	*/
# define PC_24			0x0000	/* 24 Bit Mantisse 		*/
# define PC_53			0x0200	/* 53 Bit Mantisse 		*/
# define PC_64			0x0300	/* 64 Bit Mantisse		*/

# define MCW_RC			0x0c00	/* Rundungseinstellung		*/
# define RC_NEAR		0x0000	/* zur naechsten ganzen Zahl	*/
# define RC_DOWN		0x0400	/* in Richtung -Inf		*/
# define RC_UP			0x0800	/* in Richtung +Inf		*/
# define RC_CHOP		0x0c00	/* in Richtung 0		*/

# define MCW_IC			0x1000	/* Unendlichkeit		*/
# define IC_PROJECTIVE		0x0000	/* -Inf != +INf (immer ab 387)	*/
# define IC_AFFINE		0x1000	/* -Inf == +Inf (nur bis 287)	*/

# define CW_DEFAULT		0x037f	/* Standardwert	*/

#endif	/* MCW_EM */


#ifndef SW_INVALID	/* Status Word (wie in FLOAT.H) */

# define SW_INVALID		0x0001	/* ungueltige Operation		*/
# define SW_DENORMAL		0x0002	/* nicht normale Zahl		*/
# define SW_ZERODIVIDE		0x0004	/* durch 0 geteilt		*/
# define SW_OVERFLOW		0x0008	/* Ergebnis zu gross		*/
# define SW_UNDERFLOW		0x0010	/* Ergebnis zu klein		*/
# define SW_INEXACT		0x0020	/* Ergebnis wurde gerundet	*/
# define SW_STACKFAULT		0x0040	/* Stack-Fehler (ab 387)	*/

#endif	/* SW_INVALID */

#define SW_EXEPTIONSUMMARY	0x0080	/* Fehler ist aufgetreten	*/
#define SW_C0			0x0100	/* Vergleichsflag 0		*/
#define SW_C1			0x0200	/* Vergleichsflag 1		*/
#define SW_C2			0x0400	/* Vergleichsflag 2		*/
#define SW_C3			0x4000	/* Vergleichsflag 3		*/
#define SW_BUSY			0x8000	/* FPU ist beschaeftigt		*/

#ifndef __ASSEMBLER__		/* C-Code fuer den Assembler ausblenden	*/

/*
 * In RSXNT-DLLs (Win32) werden Daten als Zeiger auf den Inhalt uebergeben :-( 
 *
 * ( Anm.: Das funktioniert aber nur bei RSXNT-Programmen!!! )
 *
 * Durch die folgende Definition koennen die Daten importiert werden:
 *
 *  #define IMPORT_RSXNT_DLL_DATA
 */
#if defined(__RSXNT__) && defined(IMPORT_RSXNT_DLL_DATA)
# define _ieeemaxf	(*_ieeemaxf)
# define onerr 		(*onerr)
# define ercnt 		(*ercnt)
# define funno		(*funno)
# define _fpustate	(*_fpustate)
# define __angle	(*__angle)
# define __elgna	(*__elgna)
# define __elgna_2	(*__elgna_2)
#endif

typedef struct _ENV387	/* Koprozessor-Umgebung 32-Bit Protected-Mode	*/
{
  unsigned short cw;		/* Control Word	Register		*/
  unsigned short reserved0;
  unsigned short sw;		/* Status Word Register			*/
  unsigned short reserved1;
  unsigned short tag;		/* Tag Word Register			*/
  unsigned short reserved2;
  unsigned long  eip;		/* Offset-Adresse des letzten Befehls	*/
  unsigned short cs;		/* Selektor des letzten Befehls		*/
  unsigned short code;		/* OpCode des letzen Befehls		*/
  unsigned long  optr;		/* Offset-Adresse des letzten Operanden	*/
  unsigned short osel;		/* Selektor des letzten Operanden	*/
  unsigned short reserved3;
} ENV387, *pENV387;

typedef struct _FPU387	/* Alle Koprozessor-Register der 387er-FPU	*/
{
  ENV387	env;		/* 387er Koprozessor-Umgebung 		*/
  unsigned char	reg[8][10];	/* 8 Koprozessor-Register		*/
} FPU387, *pFPU387;


/*
 *  Fehlerbehandlung
 */
typedef void		(*POPUPFUNC)(void), *(*pPOPUPFUNC)(void);
typedef int 		(*ERRNOFUNC)(void), *(*pERRNOFUNC)(void);

char *_errmsg(char *msg);	/* Fehlermeldung schreiben	*/
void _popup(void);		/* interne popup()-Funktion	*/
extern POPUPFUNC onerr;			/* Adresse der popup()-Funktion	*/

extern long ercnt;			/* Fehler-Zaehler		*/
extern long funno;			/* ID-Nummer der Funktion	*/
extern FPU387 _fpustate;		/* Inhalt der FPU-Register	*/

#ifdef __linux__
# define _errno		__errno_location	/* <errnos.h>	*/
#endif
int *_errno(void);			/* <errno.h>		*/
void _seterrno(pERRNOFUNC pFunction);	/* `errno'-Handler	*/
int *_geterrno(void);
const char *funcname(long ID);

/*
 *  Information
 */
typedef struct _CPUID {		/* Informationen des cpuid-Befehls */
  unsigned long  ebx;
  unsigned long  edx;		/* Register-Ordnung ist so gewollt!	*/
  unsigned long  ecx;
  unsigned char  zero;		/* ASCIIZ-'\0' */
  unsigned char  reserved;
  unsigned short maxlevel;	/* maximaler level von cpuid() */ 
  unsigned long	 eax;
} CPUID, *pCPUID;

typedef enum {
  cpu_086 = 0,	/* nicht unterstuetzt */
  cpu_186,	/* nicht unterstuetzt */
  cpu_286,	/* nicht unterstuetzt */
  cpu_386,
  cpu_486,
  cpu_586,
  cpu_686,
  cpu_786,	/* warten wir's ab... */
  cpu_886,
  cpu_986,
} CPUFAM;

typedef enum {
  fpu_087 = 0,	/* nicht unterstuetzt */
  fpu_non = 0,	/* keine FPU vorhanden */
  fpu_emu = 1,	/* den 80187 hat es nie gegeben */
  fpu_287,	/* nur teilweise unterstuetzt? */
  fpu_387,
  fpu_487,
  fpu_587,
  fpu_687,
  fpu_787,
  fpu_887,
  fpu_987,
} FPUFAM;

typedef enum			/* Modi der Winkel(umkehr)funktionen	*/
{
  am_ask	= 0,		/* nur Abfragen */
  am_rad	= 1,		/* Radiant */
  am_deg	= 2,		/* Grad */
  am_gon	= 3,		/* Gon */
} ANGLEMODE;

#ifdef __GNUC__
typedef unsigned long long	t_count;	/* Zaehler */
#endif

extern long double __angle;		/* Winkelfunktion-Steuerung	*/
extern long double __elgna;		/* Winkelfunktion-Steuerung	*/
extern long double __elgna_2;		/* Winkelfunktion-Steuerung	*/

int accura(void);
ANGLEMODE angle(ANGLEMODE mode);
double calcspeed(int loops, void (*call)(void));
t_count counter(void);
unsigned long counterfreq(void);
int cpuid(int level, pCPUID info);
CPUFAM cpufam(void);
FPUFAM fpufam(void);
const char *fpuoem(void);
unsigned long info(void);
int ispbug(void);
unsigned _fxam(double x);
unsigned _fxamf(float x);
unsigned _fxaml(long double x);
int _isfinit(double x);
int _isfinitf(float x);
int _isfinitl(long double x);
int _isinf(double x);
int _isinff(float x);
int _isinfl(long double x);
int _isnan(double x);
int _isnanf(float x);
int _isnanl(long double x);
int _isnorm(double x);
int _isnormf(float x);
int _isnorml(long double x);
int _isodd(double x);
int _isoddf(float x);
int _isoddl(long double x);
int _issign(double x);
int _issignf(float x);
int _issignl(long double x);
void __testerr(long ID);
void __cor_arg(register int ax, register long dx) __attribute__ ((regparm(2)));
void __cor_red(register int ax, register long dx) __attribute__ ((regparm(2)));
void __matherr(register int ax, register long dx) __attribute__ ((regparm(2)));

/*
 *  Koprozessor-Steuerung
 */
typedef enum {		/* Rechenzeit-Steuerung */
  prio_idle = 1,
  prio_norm = 2,
  prio_high = 4,
  prio_excl = 3,	/* Exklusiv: Vorsicht! */
} PRIORITY;
  
void priority(PRIORITY level);
unsigned _clear87(void);
unsigned _control87(unsigned bits, unsigned mask);
unsigned _fclear(void);
pENV387 _fldenv(pENV387 pEnv);
void _fpreset(void);
pFPU387 _frstor(pFPU387 pFpu);
pFPU387 _fsave(pFPU387 pFpu);
pENV387 _fstenv(pENV387 pEnv);
unsigned _status87(void);

double _test_(double dummy);


const char *fmath_verinfo(void);
extern unsigned long revnum;

#endif	/* __ASSEMBLER__ */

#if defined (__cplusplus)
}
#endif


#endif		/* FMATHCTL_H */

