/*
 * FMATHADD is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2001 by Michael Schroeder <mdhs@gmx.net>
 */


#define VERSION		"1.5.1"

#define ADDIN		"fmathadd"


#define DEBUG 

#ifdef DEBUG
# define DBGMSG(msg...)	fprintf(stderr, ADDIN ": " ##msg)  /* GNU extension */
#else
# define DBGMSG(msg...)
#endif

//DBGMSG("col1 = %d, row1 = %d, tab1 = %d, col2 = %d, row2 = %d, tab2 = %d, count = %d\n", array->col1, array->row1, array->tab1, array->col2, array->row2, array->tab2, array->count);
//DBGMSG("col = %d, row = %d, value = %f\n", cell->col, cell->row, cell->value);
//DBGMSG("col = %d, row = %d, err = %d, len = %d, string = %s\n", cell->col, cell->row, cell->error, cell->len, cell->string);

//DBGMSG("col = %d, row = %d, err = %d, type = %d, len = %d, string = %s\n", cell->col, cell->row, cell->error, cell->type, cell->var.string.len, cell->var.string.text);
    
//DBGMSG("col = %d, row = %d, err = %d, type = %d, value = %f\n", cell->col, cell->row, cell->error, cell->type, cell->var.value);



#define EPSILON		2.2204460492503131e-016	/* aus ehem. float.h */

#define INVALID		F_QNAN			/* Err:503 (ungltig) */

#define ERR_ARG(r)	strcpy(r, INVALID_ARG)
#define ERR_CAL(r)	strcpy(r, INVALID_CAL)

/*
 * Zeichen zu einer Nummer zusammenbasteln (little endian!)
 */
#define MKNUM(a,b)	((((unsigned char)b)<<8)|((unsigned char)a))

#ifdef __EMX__
# define SKIPSPACE(s)	while (isspace((int)*s)) s++
#else
# define SKIPSPACE(s)	while (isspace(*s)) s++
#endif


typedef enum
{
  FUNC_BESSEL = 0,
  FUNC_GAMMA,
  FUNC_WEBER,

  FUNC_XREAL,
  FUNC_XIMAG,
  FUNC_XABS,
  FUNC_XARG,
  FUNC_XCONJ,
  FUNC_XADD,
  FUNC_XSUB,
  FUNC_XMUL,
  FUNC_XDIV,
  FUNC_XSQRT,
  FUNC_XPOW,
  FUNC_XEXP,
  FUNC_XPOW10,
  FUNC_XLOGY,
  FUNC_XLOG,
  FUNC_XLOG10,
  FUNC_XSIN,
  FUNC_XCOS,
  FUNC_XTAN,
  FUNC_XSINH,
  FUNC_XCOSH,
  FUNC_XTANH,
  FUNC_XASIN,
  FUNC_XACOS,
  FUNC_XATAN,
  FUNC_XASINH,
  FUNC_XACOSH,
  FUNC_XATANH,
  FUNC_XSUM,

  FUNC_EURO,
  FUNC_IEEE,
  FUNC_ROMAN,

  FUNC_IPFUNC,
  FUNC_IPROOT,
  FUNC_IPEXTR,
  FUNC_LAGRANGE,
  FUNC_POLY,

  COUNT_OF_FUNCS,	/* dieser Wert ist die Anzahl aller Funktionen */ 
} FUNCS;


typedef enum
{
  DESCR( NUMBER ),
  DESCR( COMPLEX ),
  DESCR( DECIMAL ),
  DESCR( COEFF ),
  DESCR( ORDER ),
  DESCR( ROOT ),
  DESCR( ARRAY ),
  DESCR( XARRAY ),
  DESCR( ROMANS ),

  DESCR (GAMMA),
  DESCR (BESSEL),
  DESCR( WEBER ),

  DESCR( XREAL ),
  DESCR( XIMAG ),
  DESCR( XABS ),
  DESCR( XARG ),
  DESCR( XCONJ ),
  DESCR( XADD ),
  DESCR( XSUB ),
  DESCR( XMUL ),
  DESCR( XDIV ),
  DESCR( XSQRT ),
  DESCR( XPOW ),
  DESCR( XEXP ),
  DESCR( XPOW10 ),
  DESCR( XLOGY ),
  DESCR( XLOG ),
  DESCR( XLOG10 ),
  DESCR( XSIN ),
  DESCR( XCOS ),
  DESCR( XTAN ),
  DESCR( XSINH ),
  DESCR( XCOSH ),
  DESCR( XTANH ),
  DESCR( XASIN ),
  DESCR( XACOS ),
  DESCR( XATAN ),
  DESCR( XASINH ),
  DESCR( XACOSH ),
  DESCR( XATANH ),
  DESCR( XSUM ),

  DESCR( EURO ),
  DESCR( IEEE ),
  DESCR( ROMAN ),

  DESCR( IPFUNC ),
  DESCR( IPROOT ),
  DESCR( IPEXTR ),
  DESCR( LAGRANGE ),
  DESCR( POLY ),
} STUFF;


/*
 * fmath.c
 */

/*
 * analysis.c
 */
void CALLTYPE SC_gamma(double *, double *);
void CALLTYPE SC_bessel(double *, double *, double *);
void CALLTYPE SC_weber(double *r, double *, double *);


/*
 * complex.c
 */
void CALLTYPE SC_xreal(double *, char *);
void CALLTYPE SC_ximag(double *, char *);
void CALLTYPE SC_xabs(double *, char *);
void CALLTYPE SC_xarg(double *, char *);
void CALLTYPE SC_xconj(char *, char *);
void CALLTYPE SC_xadd(char *, char *, char *);
void CALLTYPE SC_xsub(char *, char *, char *);
void CALLTYPE SC_xmul(char *, char *, char *);
void CALLTYPE SC_xdiv(char *, char *, char *);
void CALLTYPE SC_xsqrt(char *, char *);
void CALLTYPE SC_xpow(char *, char *, char *);
void CALLTYPE SC_xexp(char *, char *);
void CALLTYPE SC_xpow10(char *, char *);
void CALLTYPE SC_xlogy(char *, char *, char *);
void CALLTYPE SC_xlog(char *, char *);
void CALLTYPE SC_xlog10(char *, char *);
void CALLTYPE SC_xsin(char *, char *);
void CALLTYPE SC_xcos(char *, char *);
void CALLTYPE SC_xtan(char *, char *);
void CALLTYPE SC_xsinh(char *, char *);
void CALLTYPE SC_xcosh(char *, char *);
void CALLTYPE SC_xtanh(char *, char *);
void CALLTYPE SC_xasin(char *, char *);
void CALLTYPE SC_xacos(char *, char *);
void CALLTYPE SC_xatan(char *, char *);
void CALLTYPE SC_xasinh(char *, char *);
void CALLTYPE SC_xacosh(char *, char *);
void CALLTYPE SC_xatanh(char *, char *);
void CALLTYPE SC_xsum(char *, CELLARRAY *);


/*
 * matrix.c
 */
void CALLTYPE SC_lagrange(double *, double *, NUMBERARRAY *);
void CALLTYPE SC_poly(double *, double *, NUMBERARRAY *);
void CALLTYPE SC_ipfunc(double *, double *, NUMBERARRAY *);
void CALLTYPE SC_iproot(double *, double *, NUMBERARRAY *);
void CALLTYPE SC_ipextr(double *, NUMBERARRAY *);


/*
 * misc.c
 */
void CALLTYPE SC_euro(char *, double *, double *);
void CALLTYPE SC_ieee(char *, double *);
void CALLTYPE SC_roman(double *, char *);



